"""
Functions for measuring memory usage, to allow usage to be optimized.

Examples::

  bin/python -c 'execfile("topo/misc/memuse.py") ; print topsize_mb()'
  
  ./topographica -c 'from topo.misc import memuse' -c 'print memuse.topsize_mb()'
  
  ./topographica -c 'from topo.misc import memuse, asizeof' -c 'print memuse.topsize_mb()'
  
  ./topographica -a -c 'from topo.misc import memuse, asizeof' -c 'print memuse.topsize_mb()'
  
  ./topographica -a -c 'from topo.misc import memuse, asizeof' examples/tiny.ty -c 'print memuse.allsizes_mb()'
  
  ./topographica -a -c 'from topo.misc import memuse, asizeof' -c 'memuse.memuse_batch("examples/tiny.ty",cortex_density=20)'

  ./topographica -a -c 'from topo.misc import memuse, asizeof' -c 'memuse.memuse_batch("examples/tiny.ty",times=[0,100],analysis_fn=memuse.plotting_and_saving_analysis_fn,cortex_density=20)'

$Id$
"""
__version__='$Revision: 10367 $'

# If functions in this file need anything other than the very basic
# imports declared here at the top, they must do so using import
# statements *within* the function definition to avoid polluting
# memory.  Otherwise, the simplest functions in this file would end up
# measuring memory taken by unused and irrelevant imports like 'topo',
# which doesn't ever need to be loaded for the 'execfile' example
# shown above.

import subprocess


def cmd_to_string(cmd):
    """Run a system command as in os.system(), but capture the stdout and return it as a string."""
    return subprocess.Popen(cmd,shell=True,stdout=subprocess.PIPE).communicate()[0]



def topsize():
    """Return the RES size of this process as reported by the top(1) command."""
    import os
    top_line=cmd_to_string("top -n 1 -b -p %d | grep '^[ ]*%d '" % (os.getpid(),os.getpid()))
    return top_line.split()[5]



def simsize():
    """
    Return the size of topo.sim reported by asizeof.asizeof().
    This estimate does not currently include any numpy arrays, and
    may also be missing other important items.

    Python 2.6 supports getsizeof() and a __sizeof__ attribute that user
    code can implement, which should provide a more accurate estimate.
    """
    import asizeof,topo
    return asizeof.asizeof(topo.sim)



###############################################################################
# String-formatted versions of the above

def mb(bytes):
    """Format the given value in bytes as a string in megabytes"""
    return "%dMB" % (bytes/1024.0/1024.0)

def topsize_mb():
    """String-formatted version of the RES size of this process reported by top(1)."""
    return "topsize:%s" % topsize()

def simsize_mb():
    """String-formatted version of the value reported by asizeof(topo.sim)."""
    return "simsize:%s" % (mb(simsize()))

def wtsize_mb():
    """String-formatted version of the memory taken by the weights, from print_sizes()."""
    from topo.command import n_bytes
    return "wtsize:%s" % (mb(n_bytes()))

def allsizes_mb():
    """
    Collates results from topsize, simsize, and wtsize.

    Formatted to suggest that the topsize is made up of code (not
    currently estimated), topo.sim (apart from weights), and weights.
    """
    from topo.command import n_bytes
    return "%s =? code + %s + %s (%s tot)" % (topsize_mb(),simsize_mb(),wtsize_mb(),mb(simsize()+n_bytes()))


###############################################################################
# Batch commands


def default_memuse_analysis_fn(prefix=""):
    """Basic memuse function for use with memuse_batch()"""
    import topo
    print "%st%s: %s" % (prefix,topo.sim.timestr(),allsizes_mb())



def plotting_and_saving_analysis_fn(prefix=""):
    """For use with memuse_batch() to test snapshot and plotting memory usage."""
    import topo
    from topo.command import save_snapshot
    from topo.command.analysis import measure_sine_pref,save_plotgroup
    
    print "%sMemuse at time %s: %s" % (prefix,topo.sim.timestr(),allsizes_mb())
    measure_sine_pref()
    print "%sAfter measure_sine_pref:  %s" % (prefix,allsizes_mb())
    save_plotgroup("Orientation Preference")
    print "%sAfter save_plotgroup:     %s" % (prefix,allsizes_mb())
    save_snapshot("/tmp/tmp.typ")
    print "%sAfter save_snapshot:      %s" % (prefix,allsizes_mb())



def memuse_batch(script_file,times=[0],analysis_fn=default_memuse_analysis_fn,**params):
    """
    Similar to run_batch, but analyzes the memory requirement of a simulation at different times.
    
    First, the specified script file will be run using the specified parameters.
    Then at each of the specified times, the given analysis_fn (which
    calls allsizes_mb() by default) is run.  The output is labeled
    with the script file, time, and parameters so that results from
    different runs can be compared.
    """
    import os,re,__main__,topo
    from topo.misc.commandline import global_params

    # Construct simulation name, etc.
    scriptbase= re.sub('.ty$','',os.path.basename(script_file))
    prefix = ""
    #prefix += time.strftime("%Y%m%d%H%M") + "_"
    prefix += scriptbase
    simname = prefix
    
    # Construct parameter-value portion of filename; should do more filtering
    for a,val in params.iteritems():
        # Special case to give reasonable filenames for lists
        valstr= ("_".join([str(i) for i in val]) if isinstance(val,list)
                 else str(val))
        prefix += "," + a + "=" + valstr

    # Set provided parameter values in main namespace
    global_params.set_in_context(**params)
    
    # Run script in main
    try:
        execfile(script_file,__main__.__dict__) #global_params.context
        global_params.check_for_unused_names()
        topo.sim.name=simname
    
        # Run each segment, doing the analysis and saving the script state each time
        for run_to in times:
            topo.sim.run(run_to - topo.sim.time())
            analysis_fn(prefix=prefix+": ")

    except:
        import traceback,sys
        traceback.print_exc(file=sys.stdout)
        sys.stderr.write("Warning -- Error detected: execution halted.\n")



"""
A family of function objects for changing a set of weights over time.

Learning functions come in two varieties: LearningFunction, and
CFPLearningFunction.  A LearningFunction (e.g. Hebbian) applies to one
set of weights, typically from one ConnectionField.  To apply learning
to an entire CFProjection, a LearningFunction can be plugged in to
CFPLF_Plugin.  CFPLF_Plugin is one example of a CFPLearningFunction,
which is a function that works with the entire Projection at once.
Some optimizations and algorithms can only be applied at the full
CFPLearningFn level, so there are other CFPLearningFns beyond
CFPLF_Plugin.

Any new learning functions added to this directory will automatically
become available for any model.

$Id$
"""

import param

from topo.base.functionfamily import LearningFn
# Imported here so that all learning functions will be in the same package
from topo.base.functionfamily import Hebbian,IdentityLF  # pyflakes:ignore (API import)



class Oja(LearningFn):
    """
    Oja's rule (Oja, 1982; Dayan and Abbott, 2001, equation 8.16.)

    Hebbian rule with soft multiplicative normalization, tending the
    weights toward a constant sum-squared value over time.  Thus this
    function does not normally need a separate output_fn for normalization.
    """
    
    alpha=param.Number(default=0.1,bounds=(0,None))
    def __call__(self,input_activity, unit_activity, weights, single_connection_learning_rate):
        weights += single_connection_learning_rate * (unit_activity * input_activity -
                                                      self.alpha * (unit_activity**2) * weights)

class Covariance(LearningFn):
    """
    Covariance learning rule supporting either input or unit thresholds.

    As presented by Dayan and Abbott (2001), covariance rules allow
    either potentiation or depression of the same synapse, depending
    on an activity level.  By default, this implementation follows
    Dayan and Abbott equation 8.8, with the unit_threshold determining
    the level of postsynaptic activity (activity of the target unit),
    below which LTD (depression) will occur.

    If you wish to use an input threshold as in Dayan and Abbott
    equation 8.9 instead, set unit_threshold to zero and change
    input_threshold to some positive value instead.  When both
    thresholds are zero this rule degenerates to the standard Hebbian
    rule.
    
    Requires some form of output_fn normalization for stability.
    """
    
    unit_threshold =param.Number(default=0.5,bounds=(0,None),
        doc="Threshold between LTD and LTP, applied to the activity of this unit.")
    input_threshold=param.Number(default=0.0,bounds=(0,None),
        doc="Threshold between LTD and LTP, applied to the input activity.")
    def __call__(self,input_activity, unit_activity, weights, single_connection_learning_rate):
        weights += single_connection_learning_rate * (unit_activity - self.unit_threshold) * (input_activity - self.input_threshold)

        
class CPCA(LearningFn):
    """
    CPCA (Conditional Principal Component Analysis) rule.

    (See O'Reilly and Munakata, Computational Explorations in 
    Cognitive Neuroscience, 2000, equation 4.12.)

    Increases each weight in proportion to the product of this
    neuron's activity, input activity, and connection weights.
 
    Has built-in normalization, and so does not require output_fn
    normalization for stability.  Intended to be a more biologically
    plausible version of the Oja rule.

    Submitted by Veldri Kurniawan and Lewis Ng.
    """
   
    def __call__(self,input_activity, unit_activity, weights, single_connection_learning_rate):
        """
        Update the value of the given weights matrix based on the
        input_activity matrix (of the same size as the weights matrix),
        the response of this unit (the unit_activity), and the previous weights 
        matrix governed by a per-connection learning rate.
        """

        weights += single_connection_learning_rate * unit_activity * (input_activity - weights);


class BCMFixed(LearningFn):
    """
    Bienenstock, Cooper, and Munro (1982) learning rule with a fixed threshold.

    (See Dayan and Abbott, 2001, equation 8.12) In the BCM rule,
    activities change only when there is both pre- and post-synaptic
    activity.  The full BCM rule requires a sliding threshold (see
    CFPBCM), but this version is simpler and easier to analyze.

    Requires some form of output_fn normalization for stability.
    """

    unit_threshold=param.Number(default=0.5,bounds=(0,None),doc="Threshold between LTD and LTP.")
    def __call__(self,input_activity, unit_activity, weights, single_connection_learning_rate):
        weights += single_connection_learning_rate * unit_activity * input_activity * (unit_activity-self.unit_threshold)



_public = list(set([_k for _k,_v in locals().items()
                    if isinstance(_v,type) and issubclass(_v,LearningFn)]))

# Automatically discover all .py files in this directory.
import os,fnmatch
__all__ = _public + [f.split('.py')[0] for f in os.listdir(__path__[0]) if fnmatch.fnmatch(f,'[!._]*.py')]
del f,os,fnmatch




"""
Tk-based graphical user interface (GUI).

This package implements a Topographica GUI based on the Tk toolkit,
through the Tkinter interface to Python.  The rest of Topographica
does not depend on this package or any module in it, and thus other
GUIs or other types of interfaces can also be used.

$Id$
"""

import sys
import os
import platform
import Tkinter

import paramtk

import topo

from topoconsole import TopoConsole,ControllableMenu

#### notes about tkgui ####
#
## Geometry management
# In several places we use pack() when grid() would probably be
# simpler. Check you know which fits a task better rather than copying
# existing code.
#
## Dialogs
# Don't know how to theme them (can't make them inherit from
# our own class, for instance). Consider making our own dialog
# boxes (subclass of Tkguiwindow) using transient and grab_set:
# http://thread.gmane.org/gmane.comp.python.tkinter/657/focus=659
#
## Mainloop
# Because we don't call mainloop(), it's necessary to call
# update() or update_idletasks() sometimes. Also, I think that sometimes
# current graphics do not update properly (but I'm not sure - I don't
# have a specific example yet). Need to clean up all the scattered
# update() and update_idletasks()...
#
## Window or Frame?
# Maybe one day everything (PlotGroupPanels, ParametersFrameWithApplys,
# ModelEditor, ...) will be Frames inside one master window (for
# e.g. a matlab-like workspace).  Nobody's been worrying too much
# about whether something's a Frame or a window when they've been
# implementing things, so 'close' buttons, title methods, and so on
# are a bit of a mix. This needs to be cleaned up when we have a
# final window organization method in mind.


# When not using the GUI, Topographica does not ordinarily import any of
# the classes in the separate Topographica packages. For example, none
# of the pattern types in topo.patterns is imported in Topographica by
# default.  But for the GUI, we want all such things to be available
# as lists from which the user can select.  To do this, we import all
# pattern types and other such classes here. User-defined classes will
# also appear in the GUI menus if they are derived from any class
# derived from the one specified in each widget, and imported before
# the relevant GUI window starts.
from topo.coordmapper import *  # pyflakes:ignore (see comment above)
from topo.ep import *  # pyflakes:ignore (see comment above)
from topo.learningfn import *  # pyflakes:ignore (see comment above)
from topo.transferfn import *  # pyflakes:ignore (see comment above)
from topo.pattern import *  # pyflakes:ignore (see comment above)
from topo.projection import *  # pyflakes:ignore (see comment above)
from topo.responsefn import *  # pyflakes:ignore (see comment above)
from topo.sheet import *  # pyflakes:ignore (see comment above)



##########
### Which os is being used (for gui purposes)?
#
# system_plaform can be:
# "linux"
# "mac"
# "win"
# "unknown"
#
# If you are programming tkgui and need to do something special
# for some other platform (or to further distinguish the above
# platforms), please modify this code.
#
# Right now tkgui only needs to detect if the platform is linux (do I
# mean any kind of non-OS X unix*?) or mac, because there is some
# special-purpose code for both those two: the mac code below, and the
# menu-activating code in topoconsole.  We might have some Windows-
# specific code for the window icons later on, too.
# * actually it's the window manager that's important, right?
# Does tkinter/tk itself give any useful information?
# What about root.tk.call("tk","windowingsystem")?

system_platform = 'unknown'
if platform.system()=='Linux':
    system_platform = 'linux'
elif platform.system()=='Darwin' or platform.mac_ver()[0]:
    system_platform = 'mac'
elif platform.system()=='Windows':
    system_platform = 'win'
##########



#
# Define up the right click (context menu) events. These variables can
# be appended or overridden in .topographicarc, if the user has some
# crazy input device.
#

if system_platform=='mac':
    # if it's on the mac, these are the context-menu events
    right_click_events = ['<Button-2>','<Button-3>','<Control-Button-1>']
    right_click_release_events = ['ButtonRelease-2','ButtonRelease-3','Control-ButtonRelease-1']
else:
    # everywhere else (I think) it's Button-3
    right_click_events = ['<Button-3>']
    right_click_release_events = ['ButtonRelease-3']
    

global TK_SUPPORTS_DOCK
TK_SUPPORTS_DOCK = True


# CEBALERT: this function needs some cleaning up.
# Some stuff should be moved out to paramtk,
# which could also make this file much simpler.

# gets set to the TopoConsole instance created by start.
console = None
def start(mainloop=False,banner=True):
    """
    Start Tk and read in an options_database file (if present), then
    open a TopoConsole.

    Does nothing if the method has previously been called (i.e. the
    module-level console variable is not None).

    mainloop: If True, then the command-line is frozen while the GUI
    is open.  If False, then commands can be entered at the command-line
    even while the GUI is operational.  Default is False.
    """
    global console

    ### Return immediately if console already set
    # (console itself might have been destroyed but we still want to
    # quit this function before starting another Tk instance, etc)
    if console is not None: return

    if banner: print 'Launching GUI'

    # tcl equivalent of 'if not hasattr(wm,forget)' would be better
    if system_platform=='mac' or Tkinter.TkVersion<8.5:
        global TK_SUPPORTS_DOCK
        TK_SUPPORTS_DOCK=False

    paramtk.initialize()
    paramtk.root.menubar = ControllableMenu(paramtk.root)
    paramtk.root.configure(menu=paramtk.root.menubar)
    
    # default,clam,alt,classic
    try:
        paramtk.root.tk.call("ttk::style","theme","use","classic")  
    except:
        pass

    # Try to read in options from an options_database file
    # (see http://www.itworld.com/AppDev/1243/UIR000616regex/
    # or p. 49 Grayson)
    try:
        options_database = os.path.join(sys.path[0],"topo","tkgui","options_database")
        paramtk.root.option_readfile(options_database)
        print "Read options database from",options_database
    except Tkinter.TclError:
        pass

    console = TopoConsole(paramtk.root)

    # Provide a way for other code to access the GUI when necessary
    topo.guimain=console

    
    # This alows context menus to work on the Mac.  Widget code should bind
    # contextual menus to the virtual event <<right-click>>, not
    # <Button-3>.
    console.event_add('<<right-click>>',*right_click_events)
    console.event_add('<<right-click-release>>',*right_click_release_events)

    # GUI/threads:
    # http://thread.gmane.org/gmane.comp.python.scientific.user/4153
    # (inc. ipython info)
    # (Also http://mail.python.org/pipermail/python-list/2000-January/021250.html)

    # mainloop() freezes the commandline until the GUI window exits.
    # Without this line the command-line remains responsive.
    if mainloop: paramtk.root.mainloop()







####################### 

if __name__ == '__main__':
    start(mainloop=True)





"""
See test_cfsom(self) and ImagePoster for an example of how to use and call
the PlotEngine system from a simulation.

$Id$
"""

import unittest
import random
from pprint import pprint
from math import pi

import param

import topo
from topo.pattern import Gaussian
from topo.plotting import plot
from topo.plotting.bitmap import *
from topo.base.sheet import Sheet, BoundingBox
from topo.base.simulation import *
from topo.base import patterngenerator
from topo.base.cf import CFProjection, CFSheet
from topo.learningfn.optimized import CFPLF_Hebbian_opt
from topo.sheet import *
from topo import numbergen

### Only for ImageSaver
from numpy.oldnumeric import resize,array,zeros
from topo.base.simulation import EventProcessor
from topo.misc.util import NxN
from pprint import *
import Image, ImageOps


### JABALERT: The ImageSaver class should probably be deleted,
### but it is currently used in this test.
class ImageSaver(EventProcessor):
    """

    A Sheet that receives activity matrices and saves them as bitmaps.
    Each time an ImageSaver sheet receives an input event on any input
    port, it saves it to a file.  The file name is determined by:

      <file_prefix><name>_<port>_<time>.<file_format>

    Where <name> is the name of the ImageSaver object, <port> is the
    name of the input port used, and <time> is the current simulation time.

    Parameters::
      file_prefix = (default '') A path or other prefix for the
                    filename.
      file_format = (default 'ppm') The file type to use when saving
                    the image. (can be any image format understood by PIL)
      time_format = (default '%f') The format string for the time.
      pixel_scale = (default 255) The amount to scale the
                     activity. Used as parameter to PIL's Image.putdata().
      pixel_offset = (default 0) The zero-offset for each pixel. Used as
                     parameter to PIL's Image.putdata()
                     
    """

    file_prefix = param.Parameter('')
    file_format = param.Parameter('ppm')
    time_format = param.Parameter('%f')
    pixel_scale = param.Parameter(255)
    pixel_offset = param.Parameter(0)



    def input_event(self,conn,data):

        self.verbose("Received %s  input from %s" % (NxN(data.shape),conn.src))
        self.verbose("input max value = %d" % max(data.ravel()))

        # assemble the filename
        filename = self.file_prefix + self.name
        if conn.dest_port:
            filename += "_" + str(conn.dest_port)
        filename += "_" + (self.time_format % self.simulation.time())
        filename += "." + self.file_format

        self.verbose("filename = '%s'" % filename)
        
        # make and populate the image
        im = Image.new('L',(data.shape[1],data.shape[0]))
        self.verbose("image size = %s" % NxN(im.size))
        im.putdata(data.ravel(),
                   scale=self.pixel_scale,
                   offset=self.pixel_offset)

        self.verbose("put image data.")

        #save the image
        f = open(filename,'w')
        im.save(f,self.file_format)
        f.close()



class TestCFSom(unittest.TestCase):

    def setUp(self):
        self.s = Simulation()
        self.sheet1 = Sheet()
        self.sheet2 = Sheet()


# CEBALERT: replace with an equivalent example that uses Image
##     def test_imagegenerator(self):
##         """
##         Code moved from __main__ block of cfsom.py.  Gives a tight example
##         of running a cfsom simulation.
##         """
##         from testsheetview import ImageGenerator
        
##         s = Simulation(step_mode=True)

##         ImageGenerator.nominal_density = 100
##         ImageGenerator.nominal_bounds = BoundingBox(points=((-0.8,-0.8),(0.8,0.8)))
##         input = ImageGenerator(filename='images/ellen_arthur.pgm')
    
    
##         save = ImageSaver(pixel_scale=1.5)
##         som = CFSheet()
        
##         s.add(som,input,save)
##         s.connect(input,som,connection_type=CFProjection,learning_fn=CFPLF_Hebbian_opt())
##         s.connect(som,save)
##         s.run(duration=10)
    


    def test_cfsom(self):
        """
        """

        gaussian_width = 0.02
        gaussian_height = 0.9

        input_pattern = Gaussian(
            bounds=BoundingBox(points=((-0.8,-0.8),(0.8,0.8))),
            scale=gaussian_height,
            aspect_ratio=gaussian_width/gaussian_height,
            x=numbergen.UniformRandom(lbound=-0.5,ubound=0.5,seed=100),
            y=numbergen.UniformRandom(lbound=-0.5,ubound=0.5,seed=200),
            orientation=numbergen.UniformRandom(lbound=-pi,ubound=pi,seed=300))

        
        # input generation params
        GeneratorSheet.period = 1.0
        GeneratorSheet.nominal_density = 5
        GeneratorSheet.print_level = param.parameterized.WARNING
        

        # cf som parameters
        CFSheet.nominal_density = 5

        ###########################################
        # build simulation
        
        param.parameterized.min_print_level = param.parameterized.WARNING
      
        s = Simulation()
        s.verbose("Creating simulation objects...")
        s['retina']=GeneratorSheet(input_generator=input_pattern)
        
        s['V1'] = CFSheet()
        s['V1'].print_level = param.parameterized.WARNING

        s.connect('retina','V1',delay=1,connection_type=CFProjection,
                  learning_fn=CFPLF_Hebbian_opt())
        s.print_level = param.parameterized.WARNING

        self.assertTrue(topo.sim['V1'].projections().get('retinaToV1',None) != None)
        self.assertTrue(topo.sim['V1'].projections().get('retinaToV1',None) != None)
        s.run(10)



suite = unittest.TestSuite()
suite.addTest(unittest.makeSuite(TestCFSom))



"""
Basic function objects.

TransferFns: accept and modify a 2d array
ResponseFns: accept two 2d arrays and return a scalar
LearningFns: accept two 2d arrays and a scalar, return one of the arrays modified

$Id$
"""

# CEBALERT: Documentation is just draft.

import numpy

import param

from imagen.transferfn import TransferFn, IdentityTF # pyflakes:ignore (API import)

class LearningFn(param.Parameterized):
    """
    Abstract base class for learning functions that plug into
    CFPLF_Plugin.
    """

    __abstract = True

    # JABALERT: Shouldn't the single_connection_learning_rate be
    # omitted from the call and instead made into a class parameter?
    def __call__(self,input_activity, unit_activity, weights, single_connection_learning_rate):
        """
        Apply this learning function given the input and output
        activities and current weights.
        
        Must be implemented by subclasses.
        """
        raise NotImplementedError


class Hebbian(LearningFn):
    """
    Basic Hebbian rule; Dayan and Abbott, 2001, equation 8.3.

    Increases each weight in proportion to the product of this
    neuron's activity and the input activity.
    
    Requires some form of output_fn normalization for stability.
    """
    
    def __call__(self,input_activity, unit_activity, weights, single_connection_learning_rate):
        weights += single_connection_learning_rate * unit_activity * input_activity


class IdentityLF(LearningFn):
    """
    Identity function; does not modify the weights.

    For speed, calling this function object is sometimes optimized
    away entirely.  To make this feasible, it is not allowable to
    derive other classes from this object, modify it to have different
    behavior, add side effects, or anything of that nature.
    """
    
    def __call__(self,input_activity, unit_activity, weights, single_connection_learning_rate):
        pass



class ResponseFn(param.Parameterized):
    """Abstract base class for response functions that plug into CFPRF_Plugin."""

    __abstract = True

    def __call__(self,m1,m2):
        """
        Apply the response function; must be implemented by subclasses.
        """
        raise NotImplementedError



class DotProduct(ResponseFn):
    """
    Return the sum of the element-by-element product of two 2D
    arrays.  
    """
    def __call__(self,m1,m2):
        # CBENHANCEMENT: numpy.vdot(m1,m2)?
        # Early tests indicated that
        # vdot(a,b)=dot(a.ravel(),b.ravel()) but was slower. Should
        # check that.
        return numpy.dot(m1.ravel(),m2.ravel())
        


class CoordinateMapperFn(param.Parameterized):
    """Abstract base class for functions mapping from a 2D coordinate into another one."""

    __abstract = True

    def __call__(self,x,y):
        """
        Apply the coordinate mapping function; must be implemented by subclasses.
        """
        raise NotImplementedError


class IdentityMF(CoordinateMapperFn):
    """Return the x coordinate of the given coordinate."""
    def __call__(self,x,y):
        return x,y


class PatternDrivenAnalysis(param.Parameterized):
    """
    Abstract base class for various stimulus-response types of analysis.

    This type of analysis consists of presenting a set of input
    patterns and collecting the responses to each one, which one will
    often want to do in a way that does not affect the current state
    of the network.

    To achieve this, the class defines several types of hooks where
    arbitrary function objects (i.e., callables) can be registered.
    These hooks are generally used to ensure that unrelated previous
    activity is eliminated, that subsequent patterns do not interact,
    and that the initial state is restored after analysis.

    Any subclasses must ensure that these hook lists are run at the
    appropriate stage in their processing, using e.g. 
    "for f in some_hook_list: f()".
    """
    
    __abstract = True

    pre_analysis_session_hooks = param.HookList(default=[],instantiate=False,doc="""
        List of callable objects to be run before an analysis session begins.""")
    
    pre_presentation_hooks = param.HookList(default=[],instantiate=False,doc="""
        List of callable objects to be run before each pattern is presented.""")
    
    post_presentation_hooks = param.HookList(default=[],instantiate=False,doc="""
        List of callable objects to be run after each pattern is presented.""")
    
    post_analysis_session_hooks = param.HookList(default=[],instantiate=False,doc="""
        List of callable objects to be run after an analysis session ends.""")



"""
Sheets for simulating a moving eye.

This module provides two classes, ShiftingGeneratorSheet, and
SaccadeController, that can be used to simulate a moving eye,
controlled by topographic neural activity from structures like the
superior colliculus.

ShiftingGeneratorSheet is a subclass of GeneratorSheet that accepts a
saccade command on the 'Saccade' port in the form of a tuple:
(amplitude,direction), specified in degrees.  It shifts its sheet
bounds in response to this command, keeping the centroid of the bounds
within a prespecified boundingregion.

SaccadeController is a subclass of CFSheet that accepts CF projections
and decodes its resulting activity into a saccade command suitable for
controlling a ShiftingGeneratorSheet.


$Id$
"""

from numpy import sin,cos,pi,array,asarray,argmax,zeros,\
     nonzero,take,random

import param

from topo.base.cf import CFSheet
from topo.base.simulation import PeriodicEventSequence,FunctionEvent
from topo.base.boundingregion import BoundingBox,BoundingRegionParameter
from topo.base.functionfamily import CoordinateMapperFn, IdentityMF
from topo.sheet import SequenceGeneratorSheet
from topo.misc import util


# JPALERT: The next three functions (activity_centroid,
# activity_sample, and activity_mode) could actually apply to any
# Sheet.  Maybe they should be moved to topo.base.sheet?

def activity_centroid(sheet,activity=None,threshold=0.0):
    """
    Return the sheet coords of the (weighted) centroid of sheet activity.

    If the activity argument is not None, then it is used instead
    of sheet.activity.  If the sheet activity is all zero, the
    centroid of the sheet bounds is returned.
    """

    if activity is None:
        activity = sheet.activity

    ys = sheet.sheet_rows()
    xs = sheet.sheet_cols()

    xy = array([(x,y) for y in reversed(ys) for x in xs])
    a = activity.flat

    ## Optimization to only compute centroid from
    ## active (non-zero) units. 
    idxs = nonzero(a > threshold)[0]
    if not len(idxs):
        return sheet.bounds.centroid()
    return util.centroid(take(xy,idxs,axis=0),take(a,idxs))


def activity_sample(sheet,activity=None):
    """
    Sample from the sheet activity as if it were a probability distribution.

    Returns the sheet coordinates of the sampled unit.  If
    activity is not None, it is used instead of sheet.activity.
    """

    if activity is None:
        activity = sheet.activity
    idx = util.weighted_sample_idx(activity.ravel())
    r,c = util.idx2rowcol(idx,activity.shape)

    return sheet.matrix2sheet(r,c)


def activity_mode(sheet,activity=None):
    """
    Returns the sheet coordinates of the mode (highest value) of
    the sheet activity.   
    """

    # JPHACKALERT:  The mode is computed using numpy.argmax, and
    # thus for distributions with multiple equal-valued modes, the
    # result will have a systematic bias toward higher x and lower
    # y values. (in that order).  Function may still be useful for
    # unimodal activity distributions, or sheets without limiting/squashing
    # output functions.
    if activity is None:
        activity = sheet.activity
    idx = argmax(activity.flat)
    r,c = util.idx2rowcol(idx,activity.shape)
    return sheet.matrix2sheet(r,c)
      


class SaccadeController(CFSheet):
    """
    Sheet that decodes activity on CFProjections into a saccade command.
    
    This class accepts CF-projected input and computes its activity
    like a normal CFSheet, then decodes that activity into a saccade
    amplitude and direction as would be specified by activity in the
    superior colliculi.  The X dimension of activity corresponds to
    amplitude, the Y dimension to direction.  The activity is decoded
    to a single (x,y) point according to the parameter decode_method.

    From this (x,y) point an (amplitude,direction) pair, specified in
    degrees, is computed using the parameters amplitude_scale and
    direction scale.  That pair is then sent out on the 'Saccade'
    output port.

    NOTE: Non-linear mappings for saccade commands, as in Ottes, et
    al (below), are assumed to be provided using the coord_mapperg
    parameter of the incoming CFProjection.

    References:
    Ottes, van Gisbergen, Egglermont. 1986.  Visuomotor fields of the
    superior colliculus: a quantitative model.  Vision Research;
    26(6): 857-73.
    """

    # JPALERT: amplitude_scale and direction scale can be implemented as
    # part of self.command_mapper, so these should probably be removed.

    amplitude_scale = param.Number(default=120,doc="""
        Scale factor for saccade command amplitude, expressed in
        degrees per unit of sheet.  Indicates how large a saccade is
        represented by the x-component of the command input.""")
    
    direction_scale = param.Number(default=180,doc="""
        Scale factor for saccade command direction, expressed in
        degrees per unit of sheet.  Indicates what direction of saccade
        is represented by the y-component of the command input.""")


    decode_fn = param.Callable(default=activity_centroid,
                                  instantiate=False,doc="""
        The function for extracting a single point from sheet activity.
        Should take a sheet as the first argument, and return (x,y).""")

    command_mapper = param.ClassSelector(CoordinateMapperFn,default=IdentityMF(),
                                   doc="""
        A CoordinateMapperFn that will be applied to the command vector extracted
        from the sheet activity.""")

    src_ports = ['Activity','Saccade']


    def activate(self):
        super(SaccadeController,self).activate()
            
        # get the input projection activity            
        # decode the input projection activity as a command
        xa,ya = self.decode_fn(self)
        self.verbose("Saccade command centroid = (%.2f,%.2f)"%(xa,ya))

        xa,ya = self.command_mapper(xa,ya)
        
        amplitude = xa * self.amplitude_scale
        direction = ya * self.direction_scale

        self.verbose("Saccade amplitute = %.2f."%amplitude)
        self.verbose("Saccade direction = %.2f."%direction)
        
        self.send_output(src_port='Saccade',data=(amplitude,direction))




        
class ShiftingGeneratorSheet(SequenceGeneratorSheet):
    """
    A GeneratorSheet that takes an extra input on port 'Saccade'
    that specifies a saccade command as a tuple (amplitude,direction),
    indicating the relative size and direction of the saccade in
    degrees.  The parameter visual_angle_scale defines the
    relationship between degrees and sheet coordinates.  The parameter
    saccade bounds limits the region within which the saccades may occur.
    """

    visual_angle_scale = param.Number(default=90,doc="""
        The scale factor determining the visual angle subtended by this sheet, in
        degrees per unit of sheet.""")

    saccade_bounds = BoundingRegionParameter(default=BoundingBox(radius=1.0),doc="""
        The bounds for saccades.  Saccades are constrained such that the centroid of the
        sheet bounds remains within this region.""")

    generate_on_shift = param.Boolean(default=True,doc="""
       Whether to generate a new pattern when a shift occurs.""")
                                         
    fixation_jitter = param.Number(default=0,doc="""
       Standard deviation of Gaussian fixation jitter.""")
    fixation_jitter_period = param.Number(default=10,doc="""
       Period, in time units, indicating how often the eye jitters.
       """)

    dest_ports = ["Trigger","Saccade"]
    src_ports = ['Activity','Position']

    def __init__(self,**params):
        super(ShiftingGeneratorSheet,self).__init__(**params)
        self.fixation_point = self.bounds.centroid()

    def start(self):
        super(ShiftingGeneratorSheet,self).start()
        if self.fixation_jitter_period > 0:
            now = self.simulation.time()
            refix_event = PeriodicEventSequence(now+self.simulation.convert_to_time_type(self.fixation_jitter_period),
                                                self.simulation.convert_to_time_type(self.fixation_jitter_period),
                                                [FunctionEvent(0,self.refixate)])
            self.simulation.enqueue_event(refix_event)

    def input_event(self,conn,data):
        if conn.dest_port == 'Saccade':
            # the data should be (amplitude,direction)
            amplitude,direction = data
            self.shift(amplitude,direction)

    def generate(self):
        super(ShiftingGeneratorSheet,self).generate()
        self.send_output(src_port='Position',
                         data=self.bounds.aarect().centroid())
        

    def shift(self,amplitude,direction,generate=None):
        """
        Shift the bounding box by the given amplitude and
        direction.

        Amplitude and direction are specified in degrees, and will be
        converted using the sheet's visual_angle_scale
        parameter. Negative directions are always downward, regardless
        of whether the amplitude is positive (rightword) or negative
        (leftward).  I.e. straight-down = -90, straight up = +90.

        The generate argument indicates whether or not to generate
        output after shifting.  If generate is None, then the value of
        the sheet's generate_on_shift parameter will be used.
        """
        
        # JPALERT:  Right now this method assumes that we're doing
        # colliculus-style saccades. i.e. amplitude and direction
        # relative to the current position.  Technically it would
        # not be hard to also support absolute or relative x,y
        # commands, and select what style to use with either with
        # a parameter, or with a different input port (e.g. 'xy
        # relative', 'xy absolute' etc.

        # JPHACKALERT: Currently there is no support for modeling the
        # fact that saccades actually take time, and larger amplitudes
        # take more time than small amplitudes.  No clue if we should
        # do that, or how, or what gets sent out while the saccade
        # "eye" is moving.

        assert not self._out_of_bounds()

        # convert the command to x/y translation
        radius = amplitude/self.visual_angle_scale

        # if the amplitude is negative, negate the direction (so up is still up)
        if radius < 0.0:
            direction *= -1

        self._translate(radius,direction)

        if self._out_of_bounds():
            self._find_saccade_in_bounds(radius,direction)

        self.fixation_point = self.bounds.centroid()
        
        if generate is None:
            generate = self.generate_on_shift
            
        if generate:
            self.generate()

    def refixate(self):
        """
        Move the bounds toward the fixation point.

        Moves the bounds toward the fixation point specified in
        self.fixation_point, potentially with noise as specified by
        the parameter self.fixation_jitter.
        """
        self.debug("Refixating.")

        if self.fixation_jitter > 0:
            jitter_vec = random.normal(0,self.fixation_jitter,(2,))
        else:
            jitter_vec = zeros((2,))
        
        fix = asarray(self.fixation_point)
        pos = asarray(self.bounds.centroid())
        refix_vec = (fix - pos) + jitter_vec
        self.bounds.translate(*refix_vec)

    def _translate(self,radius,angle):
        angle *= pi/180
        xoff = radius * cos(angle)
        yoff = radius * sin(angle)

        self.verbose("Applying translation vector (%.2f,%.2f)"%(xoff,yoff))
        self.bounds.translate(xoff,yoff)


    def _out_of_bounds(self):
        """
        Return true if the centroid of the current bounds is outside the saccade bounds.
        """
        return not self.saccade_bounds.contains(*self.bounds.aarect().centroid())


    def _find_saccade_in_bounds(self,radius,theta):
        """
        Find a saccade in the given direction (theta) that lies within self.saccade_bounds.

        Assumes that the given saccade was already performed and
        landed out of bounds.
        """

        # JPHACKALERT: This method iterates to search for a saccade
        # that lies in bounds along the saccade vector. We should
        # really compute this algebraically. Doing so involves computing
        # the intersection of the saccade vector with the saccade
        # bounds.  Ideally, each type of BoundingRegion would know how
        # to compute its own intersection with a given line (should be
        # easy for boxes, circles, and ellipses, at least.)

        # Assume we're starting out of bounds, so start by shifting
        # back to the original position        
        self._translate(-radius,theta)

        while not self._out_of_bounds():
            radius *= 0.5
            self._translate(radius,theta)

        radius = -radius
        while self._out_of_bounds():
            radius *= 0.5
            self._translate(radius,theta)


__all__ = [
    "SaccadeController",
    "ShiftingGeneratorSheet",
    "activity_centroid",
    "activity_sample",
    "activity_mode",
]



"""
Interface class for inline C/C++ functions.  Based on the SciPy Weave package.

Weave (from SciPy) allows programmers to implement Python methods or
functions using C code written as a string in the Python file.  This
is generally done to speed up code that would be slow if written
directly in Python.  Because not all users can be assumed to have a
working C/C++ compiler, it is crucial for such optimizations to be
optional.  This file provides an interface for processing the inline
C/C++ code in a way that can gracefully revert back to the
unoptimized version when Weave is not available.

The fallback is implemented by making use of the way Python allows
function names to be overwritten, as in these simple examples:

  def x(y = 5): return y
  def x2(y = 6): return y*y
  print 'x:', x(), 'x2:', x2()  # Result -- x: 5 x2: 36
  x = x2
  print 'x:', x(), 'x2:', x2()  # Result -- x: 36 x2: 36

In this file, inline() is overwritten to call inline_weave() if Weave
is available.  If Weave is not available, inline() will raise a
NotImplementedError exception.  For a program to be usable without
Weave, just test inlinec.optimized after defining each optimized
component, replacing it with a non-optimized equivalent if
inlinec.optimized is False.

For more information on weave, see:
http://old.scipy.org/documentation/weave/weaveusersguide.html

Some of the C functions also support OpenMP, which allows them to use
multiple threads automatically on multi-core machines to give better
performance.  To enable OpenMP support for those functions, set
openmp=True in the main namespace before importing this file, and
(optionally) set openmp_threads to the number of threads desired.  If
openmp_threads is not set, then a thread will be allocated for each
available core by default.

Note that in order to use OpenMP, the C functions are obliged to use the
thread-safe portions of the Python/Numpy C API. In general, the Python
C API cannot be assumed to be thread safe.  Calls to PyObject_SetAttrString
are a common hazard which can often be avoided using LOOKUP_FROM_SLOT_OFFSET.
This makes use of Python's __slots__ mechanism with the added benefit of
bypassing the GIL.

$Id$
"""

import collections
import os
from copy import copy

# If import_weave is not defined, or is set to True, will attempt to
# import weave.  Set import_weave to False if you want to avoid weave
# altogether, e.g. if your installation is broken.
import __main__
import_weave = __main__.__dict__.get('import_weave',True)

# Dictionary of strings used to allow optional substituions
# (e.g. pragmas) into inline C code.
#
# CB: default value is empty string for convenience of not
# having to do e.g.
# if openmp:
#     c_decorators['xyz']='abc'
# else:
#     c_decorators['xyz']=''
c_decorators = collections.defaultdict(lambda:'')

# Setting to true will cause OpenMP to be used while compiling some C
# code (search for cfs_loop_pragma to see which routines use
# OpenMP). Good multi-threaded performance requires a machine with
# separate memory subsystems for each core, such as a Xeon. See Marco
# Elver's report at http://homepages.inf.ed.ac.uk/s0787712/stuff/melver_project-report.pdf.
openmp_threads = __main__.__dict__.get('openmp_threads',False)
    
# Variable that will be used to report whether weave was successfully
# imported (below).
weave_imported = False

# Variable that will be used to report whether simple compilation test
# was successful.
compiled = False

def inline(*params,**nparams): raise NotImplementedError



##########
# Windows: hack to allow weave to work when a user name contains a
# space (see
# http://thread.gmane.org/gmane.comp.python.scientific.devel/14275)
if import_weave:
    import sys
    if sys.platform.startswith("win"):
        try:
            # catch the initial use of USERNAME by weave
            original_user = os.environ.get("USERNAME")
            os.environ["USERNAME"]=original_user.replace(" ","")
            # now dynamically patch weave and restore USERNAME
            import scipy.weave.catalog
            iam = scipy.weave.catalog.whoami().replace(" ","")
            scipy.weave.catalog.whoami = lambda: iam                
            os.environ["USERNAME"]=original_user        
        except:
            pass
##########

try:
    if import_weave:
        # We supply weave separately with the source distribution, but
        # e.g. the ubuntu package uses scipy.
        try:
            import weave
        except ImportError:
            from scipy import weave  # pyflakes:ignore (try/except import)

        weave_imported = True

    # Default parameters to add to the inline_weave() call.
    inline_named_params = {
        'extra_compile_args':['-O2','-Wno-unused-variable -fomit-frame-pointer','-funroll-loops'],
        'extra_link_args':['-lstdc++'],
        'compiler':'gcc',
        'verbose':0}

    if openmp_threads != 1:
        c_decorators['cfs_loop_pragma']="#pragma omp parallel for schedule(guided, 8)"
        inline_named_params['extra_compile_args'].append('-fopenmp')
        inline_named_params['extra_link_args'].append('-fopenmp')


    def inline_weave(*params,**nparams):
        named_params = copy(inline_named_params) # Make copy of defaults.
        named_params.update(nparams)             # Add newly passed named parameters.
        weave.inline(*params,**named_params)
        
    # Overwrites stub definition with full Weave definition
    inline = inline_weave

except ImportError:
    # CEBALERT: where does 'caution' fit in our warnings system? (Also
    # used in other places in this file.)
    print 'Caution: Unable to import Weave.  Will use non-optimized versions of most components.'


if weave_imported:
    import random
    try:
        # to force recompilation each time
        inline('double x=%s;'%random.random())
        compiled = True
    except Exception, e:
        print "Caution: Unable to use Weave to compile: \"%s\". Will use non-optimized versions of most components."%str(e)

# Flag available for all to use to test whether to use the inline
# versions or not.
optimized = weave_imported and compiled

warn_for_each_unoptimized_component = False


# JABALERT: I can't see any reason why this function accepts names rather
# than the more pythonic option of accepting objects, from which names
# can be extracted if necessary.
def provide_unoptimized_equivalent(optimized_name, unoptimized_name, local_dict):
    """
    If not using optimization, replace the optimized component with its unoptimized equivalent.
    
    The objects named by optimized_name and unoptimized_name should be
    plug-compatible.  The local_dict argument should be given the
    contents of locals(), so that this function can replace the
    optimized version with the unoptimized one in the namespace from
    which it has been called.

    As an example, calling this function as::
    
      provide_unoptimized_equivalent("sort_opt","sort",locals())

    is equivalent to putting the following code directly into the
    calling location::
    
      if not optimized:
        sort_opt = sort
        print 'module: Inline-optimized components not available; using sort instead of sort_opt.'
    """
    if not optimized:
        local_dict[optimized_name] = local_dict[unoptimized_name]
        if warn_for_each_unoptimized_component:
            print '%s: Inline-optimized components not available; using %s instead of %s.' \
                  % (local_dict['__name__'], optimized_name, unoptimized_name)

if not optimized and not warn_for_each_unoptimized_component:
    print "Note: Inline-optimized components are currently disabled; see topo.misc.inlinec"


# Definitions useful for working with optimized Python code;
# prepend to the code for an inlinec call if you want to use them.
c_header = """
/* Declaration for interfacing to numpy floats */
typedef double npfloat;

/* For a given class cls and an attribute attr, defines a variable
   attr_offset containing the offset of that attribute in the class's
   __slots__ data structure. */
#define DECLARE_SLOT_OFFSET(attr,cls) \
  PyMemberDescrObject *attr ## _descr = (PyMemberDescrObject *)PyObject_GetAttrString(cls,#attr); \
  Py_ssize_t attr ## _offset = attr ## _descr->d_member->offset; \
  Py_DECREF(attr ## _descr)

/* After a previous declaration of DECLARE_SLOT_OFFSET, for an
   instance obj of that class and the given attr, retrieves the value
   of that attribute from its slot. */
#define LOOKUP_FROM_SLOT_OFFSET(type,attr,obj) \
  PyArrayObject *attr ## _obj = *((PyArrayObject **)((char *)obj + attr ## _offset)); \
  type *attr = (type *)(attr ## _obj->data)

/* LOOKUP_FROM_SLOT_OFFSET without declaring data variable */
#define LOOKUP_FROM_SLOT_OFFSET_UNDECL_DATA(type,attr,obj) \
  PyArrayObject *attr ## _obj = *((PyArrayObject **)((char *)obj + attr ## _offset));

/* Same as LOOKUP_FROM_SLOT_OFFSET but ensures the array is contiguous.
   Must call DECREF_CONTIGUOUS_ARRAY(attr) to release temporary.
   Does PyArray_FLOAT need to be an argument for this to work with doubles? */

// This code is optimized for contiguous arrays, which are typical,
// but we make it work for noncontiguous arrays (e.g. views) by
// creating a contiguous copy if necessary.
//
// CEBALERT: I think there are better alternatives
// e.g. PyArray_GETCONTIGUOUS (PyArrayObject*) (PyObject* op)
// (p248 of numpybook), which only acts if necessary...
// Do we have a case where we know this code is being
// called, so that I can test it easily?

// CEBALERT: weights_obj appears below. Doesn't that mean this thing
// will only work when attr is weights?

#define CONTIGUOUS_ARRAY_FROM_SLOT_OFFSET(type,attr,obj) \
  PyArrayObject *attr ## _obj = *((PyArrayObject **)((char *)obj + attr ## _offset)); \
  type *attr = 0; \
  PyArrayObject * attr ## _array = 0; \
  if(PyArray_ISCONTIGUOUS(weights_obj)) \
      attr = (type *)(attr ## _obj->data); \
  else { \
      attr ## _array = (PyArrayObject*) PyArray_ContiguousFromObject((PyObject*)attr ## _obj,PyArray_FLOAT,2,2); \
      attr = (type *) attr ## _array->data; \
  }

#define DECREF_CONTIGUOUS_ARRAY(attr) \
   if(attr ## _array != 0) { \
       Py_DECREF(attr ## _array); }



#define UNPACK_FOUR_TUPLE(type,i1,i2,i3,i4,tuple) \
  type i1 = *tuple++; \
  type i2 = *tuple++; \
  type i3 = *tuple++; \
  type i4 = *tuple
"""

# Simple test
if __name__ == '__main__':
    inline('printf("Hello World!!\\n");')
    




"""
Distribution class

$Id$
"""
# To do:
#
# - wrap bins for cyclic histograms
# - check use of float() in count_mag() etc
# - clarify comment about negative selectivity
#
# - function to return value in a range (like a real histogram)
# - cache values
# - assumes cyclic axes start at 0: include a shift based on range
#
# - is there a way to make this work for arrays without mentioning
#   "array" anywhere in here?
# - should this be two classes: one for the core (which would be
#   small though) and another for statistics?



# The basic functions do not have any dependencies, but these imports
# are needed for some of the statistical functions (e.g. vector sum).
import numpy
import param

from math import pi

unavailable_scipy_optimize  = False
try:
    from scipy            import optimize
except ImportError:
    param.Parameterized().debug("scipy.optimize not available, dummy von Mises fit")
    unavailable_scipy_optimize  = True

from numpy            import cos, log
from numpy.oldnumeric import innerproduct, array, exp, argmax

from topo.base.arrayutil import arg, wrap

class Distribution(object):
    """
    Holds a distribution of the values f(x) associated with a variable x.
    
    A Distribution is a histogram-like object that is a dictionary of
    samples.  Each sample is an x:f(x) pair, where x is called the bin
    and f(x) is called the value(). Each bin's value is typically
    maintained as the sum of all the values that have been placed into
    it.

    The bin axis is continuous, and can represent a continuous
    quantity without discretization.  Alternatively, this class can be
    used as a traditional histogram by either discretizing the bin
    number before adding each sample, or by binning the values in the
    final Distribution.

    Distributions are bounded by the specified axis_bounds, and can
    either be cyclic (like directions or hues) or non-cyclic.  For
    cyclic distributions, samples provided outside the axis_bounds
    will be wrapped back into the bound range, as is appropriate for
    quantities like directions.  For non-cyclic distributions,
    providing samples outside the axis_bounds will result in a
    ValueError.

    In addition to the values, can also return the counts, i.e., the
    number of times that a sample has been added with the given bin.

    Not all instances of this class will be a true distribution in the
    mathematical sense; e.g. the values will have to be normalized
    before they can be considered a probability distribution.

    If keep_peak=True, the value stored in each bin will be the
    maximum of all values ever added, instead of the sum.  The
    distribution will thus be a record of the maximum value
    seen at each bin, also known as an envelope.
    """

    # Holds the number of times that undefined values have been
    # returned from calculations for any instance of this class,
    # e.g. calls to vector_direction() or vector_selectivity() when no
    # value is non-zero.  Useful for warning users when the values are
    # not meaningful.
    undefined_vals  = 0 

    def __init__(self, axis_bounds=(0.0, 2*pi), cyclic=False, keep_peak=False):
        self._data = {}
        self._counts = {}
            
        # total_count and total_value hold the total number and sum
        # (respectively) of values that have ever been provided for
        # each bin.  For a simple distribution these will be the same as
        # sum_counts() and sum_values().
        self.total_count = 0
        self.total_value = 0.0
        
        self.axis_bounds = axis_bounds
        self.axis_range = axis_bounds[1] - axis_bounds[0]
        self.cyclic = cyclic
        self.keep_peak = keep_peak



    ### JABHACKALERT!  The semantics of this operation are incorrect, because
    ### an expression like x+y should not modify x, while this does.  It could
    ### be renamed __iadd__, to implement += (which has the correct semantics),
    ### but it's not yet clear how to do that.
    def __add__(self,a):
        """
        Allows add() method to be used via the '+' operator; i.e.,
        Distribution + dictionary does Distribution.add(dictionary).
        """
        self.add(a)
        return None
    

    def get_value(self, bin):
        """
        Return the value of the specified bin.

        (Return None if there is no such bin.)        
        """
        return self._data.get(bin)


    def get_count(self, bin):
        """
        Return the count from the specified bin.

        (Return None if there is no such bin.)        
        """
        return self._counts.get(bin)
    

    def values(self):
        """
        Return a list of values.

        Various statistics can then be calculated if desired:
        
          sum(vals)  (total of all values)
          max(vals)  (highest value in any bin)

        Note that the bin-order of values returned does not necessarily
        match that returned by counts().
        """
        return self._data.values()


    def counts(self):
        """
        Return a list of values.

        Various statistics can then be calculated if desired:

          sum(counts)  (total of all counts)
          max(counts)  (highest count in any bin)

        Note that the bin-order of values returned does not necessarily
        match that returned by values().
        """
        return self._counts.values()


    def bins(self):
        """
        Return a list of bins that have been populated.
        """
        return self._data.keys()


    def add(self, new_data):
        """
        Add a set of new data in the form of a dictionary of (bin,
        value) pairs.  If the bin already exists, the value is added
        to the current value.  If the bin doesn't exist, one is created
        with that value.

        Bin numbers outside axis_bounds are allowed for cyclic=True,
        but otherwise a ValueError is raised.

        If keep_peak=True, the value of the bin is the maximum of the
        current value and the supplied value.  That is, the bin stores
        the peak value seen so far.  Note that each call will increase
        the total_value and total_count (and thus decrease the
        value_mag() and count_mag()) even if the value doesn't happen
        to be the maximum seen so far, since each data point still
        helps improve the sampling and thus the confidence.
        """
        for bin in new_data.keys():

            if self.cyclic==False:
                if not (self.axis_bounds[0] <= bin <= self.axis_bounds[1]):
                    raise ValueError("Bin outside bounds.")
            # CEBALERT: Neet to support wrapping of bin values
            # else:  new_bin = wrap(self.axis_bounds[0], self.axis_bounds[1], bin)
            new_bin = bin
            
            if new_bin not in self._data:
                self._data[new_bin] = 0.0
                self._counts[new_bin] = 0

            new_value = new_data[bin]
            self.total_value += new_value
            self._counts[new_bin] += 1
            self.total_count += 1

            if self.keep_peak == True:
                if new_value > self._data[new_bin]: self._data[new_bin] = new_value
            else:
                self._data[new_bin] += new_value
               

    def sub_distr( self, distr ):
        """
        Subtract the given distribution from the current one.
        Only existing bins are modified, new bins in the given
        distribution are discarded without raising errors.

        Note that total_value and total_count are not affected, and
        keep_peak is ignored, therefore analysis relying on these
        values should not call this method.
        """
        for b in distr.bins():
            if b in self.bins():
                v   = distr._data.get( b )
                if v is not None:   self._data[ b ] -= v
               

    def max_value_bin(self):
        """Return the bin with the largest value."""
        return self._data.keys()[argmax(self._data.values())]

    def weighted_sum(self):
        """Return the sum of each value times its bin."""
        return innerproduct(self._data.keys(), self._data.values()) 


    def value_mag(self, bin):
        """Return the value of a single bin as a proportion of total_value."""
        return self._safe_divide(self._data.get(bin), self.total_value)


    def count_mag(self, bin):
        """Return the count of a single bin as a proportion of total_count."""
        return self._safe_divide(float(self._counts.get(bin)), float(self.total_count))
        # use of float()


    def _bins_to_radians(self, bin):
        """
        Convert a bin number to a direction in radians.

        Works for NumPy arrays of bin numbers, returning
        an array of directions.
        """
        return (2*pi)*bin/self.axis_range 


    def _radians_to_bins(self, direction):
        """
        Convert a direction in radians into a bin number.

        Works for NumPy arrays of direction, returning
        an array of bin numbers.
        """
        return direction*self.axis_range/(2*pi)


    def _safe_divide(self,numerator,denominator):
        """
        Division routine that avoids division-by-zero errors
        (returning zero in such cases) but keeps track of them
        for undefined_values().
        """
        if denominator==0:
            self.undefined_vals += 1
            return 0
        else:
            return numerator/denominator


class Pref( dict ):
    """
    This class simply collects named arguments into a dictionary

    the main purpose is to make pretty readable the output of DistributionStatisticFn
    functions.
    In addition, trap missing keys
    """

    def __init__( self, **args ):
        dict.__init__( self, **args )

    def __getitem__( self, key ):
        try:
            return dict.__getitem__( self, key )
        except KeyError:
            return None



class DistributionStatisticFn( param.Parameterized ):
    """
    Base class for various functions performing statistics on a distribution

    """

    value_scale = param.NumericTuple( (0.0, 1.0), doc="""
            Scaling of the resulting value of the distribution statistics,
            typically the preference of a unit to feature values. The tuple
            specifies (offset, multiplier) of the output scaling""" )

# APNOTE: previously selectivity_scale[ 1 ] used to be 17, a value suitable
# for combining preference and selectivity in HSV plots. Users wishing to keep
# this value should now set it when creating SheetViews, in commands like that
# in command/analysis.py
    selectivity_scale = param.NumericTuple( (0.0, 1.0), doc="""
            Scaling of the resulting measure of the distribution peakedness,
            typically the selectivity of a unit to its preferred feature value.
            The tuple specifies (offset, multiplier) of the output scaling""" )
    
    __abstract = True

    def __call__(self, distribution):
        """
        Apply the distribution statistic function; must be implemented by subclasses.

        Subclasses sould be called with a Distribution as argument, return will be a
        dictionary, with Pref objects as values
        """
        raise NotImplementedError


class DescriptiveStatisticFn( DistributionStatisticFn ):
    """
    Abstract class for basic descriptive statistics

    """

    def vector_sum(self, d ):
        """
        Return the vector sum of the distribution as a tuple (magnitude, avgbinnum).

        Each bin contributes a vector of length equal to its value, at
        a direction corresponding to the bin number.  Specifically,
        the total bin number range is mapped into a direction range
        [0,2pi].
        
        For a cyclic distribution, the avgbinnum will be a continuous
        measure analogous to the max_value_bin() of the distribution.
        But this quantity has more precision than max_value_bin()
        because it is computed from the entire distribution instead of
        just the peak bin.  However, it is likely to be useful only
        for uniform or very dense sampling; with sparse, non-uniform
        sampling the estimates will be biased significantly by the
        particular samples chosen.

        The avgbinnum is not meaningful when the magnitude is 0,
        because a zero-length vector has no direction.  To find out
        whether such cases occurred, you can compare the value of
        undefined_vals before and after a series of calls to this
        function.

        """
        # vectors are represented in polar form as complex numbers
        h   = d._data
        r   = h.values()                                  
        theta = d._bins_to_radians(array( h.keys() ))
        v_sum = innerproduct(r, exp(theta*1j))                  

        magnitude = abs(v_sum)
        direction = arg(v_sum)

        if v_sum == 0:
            d.undefined_vals += 1

        direction_radians = d._radians_to_bins(direction)

        # wrap the direction because arctan2 returns principal values
        wrapped_direction = wrap(d.axis_bounds[0], d.axis_bounds[1], direction_radians)
        
        return (magnitude, wrapped_direction) 


    def _weighted_average(self, d ):
        """
        Return the weighted_sum divided by the sum of the values
        """
        return d._safe_divide(d.weighted_sum(),sum(d._data.values())) 


    def selectivity(self, d):
        """
        Return a measure of the peakedness of the distribution.  The
        calculation differs depending on whether this is a cyclic
        variable.  For a cyclic variable, returns the magnitude of the
        vector_sum() divided by the sum_value() (see
        _vector_selectivity for more details).  For a non-cyclic
        variable, returns the max_value_bin()) as a proportion of the
        sum_value() (see _relative_selectivity for more details).
        """
        if d.cyclic == True:
            return self._vector_selectivity( d )
        else:
            return self._relative_selectivity( d )


    # CEBHACKALERT: the definition of selectivity for non-cyclic
    # quantities probably needs some more thought.
    # Additionally, this fails the test in testfeaturemap
    # (see the comment there).
    def _relative_selectivity( self, d ):
        """
        Return max_value_bin()) as a proportion of the sum_value().

        This quantity is a measure of how strongly the distribution is
        biased towards the max_value_bin().  For a smooth,
        single-lobed distribution with an inclusive, non-cyclic range,
        this quantity is an analog to vector_selectivity.  To be a
        precise analog for arbitrary distributions, it would need to
        compute some measure of the selectivity that works like the
        weighted_average() instead of the max_value_bin().  The result
        is scaled such that if all bins are identical, the selectivity
        is 0.0, and if all bins but one are zero, the selectivity is
        1.0.
        """
        # A single bin is considered fully selective (but could also
        # arguably be considered fully unselective)
        if len(d._data) <= 1: 
            return 1.0

        proportion = d._safe_divide( max(d._data.values()),
                                        sum(d._data.values()) )
        offset = 1.0/len(d._data)
        scaled = (proportion-offset)/(1.0-offset)

        # negative scaled is possible 
        # e.g. 2 bins, with values that sum to less than 0.5
        # this probably isn't what should be done in those cases
        if scaled >= 0.0:
            return scaled
        else:
            return 0.0


    def _vector_selectivity( self, d ):
        """
        Return the magnitude of the vector_sum() divided by the sum_value().

        This quantity is a vector-based measure of the peakedness of
        the distribution.  If only a single bin has a non-zero value(),
        the selectivity will be 1.0, and if all bins have the same
        value() then the selectivity will be 0.0.  Other distributions
        will result in intermediate values.

        For a distribution with a sum_value() of zero (i.e. all bins
        empty), the selectivity is undefined.  Assuming that one will
        usually be looking for high selectivity, we return zero in such
        a case so that high selectivity will not mistakenly be claimed.
        To find out whether such cases occurred, you can compare the
        value of undefined_values() before and after a series of
        calls to this function.
        """
        return d._safe_divide(self.vector_sum( d )[0], sum(d._data.values()))

    
    __abstract = True



class DescriptiveBimodalStatisticFn( DescriptiveStatisticFn ):
    """
    Abstract class for descriptive statistics of two-modes distributions

    """

    def second_max_value_bin( self, d ):
        """
        Return the bin with the second largest value.
        If there is one bin only, return it. This is not a correct result,
        however it is practical for plotting compatibility, and it will not
        mistakenly be claimed as secondary maximum, by forcing its selectivity
        to 0.0
        """
        h   = d._data
        if len( h ) <= 1: 
            return h.keys()[ 0 ]

        k       = self.max_value_bin()
        v       = h.pop(k)
        m       = self.max_value_bin()
        h[ k ]  = v

        return m


    def second_selectivity( self, d ):
        """
        Return the selectivity of the second largest value in the distribution.
        If there is one bin only, the selectivity is 0, since there is no second
        peack at all, and this value is also used to discriminate the validity
        of second_max_value_bin()
        Selectivity is computed in two ways depending on whether the variable is
        a cyclic, as in selectivity()
        """
        if len( d._data ) <= 1: 
            return 0.0
        if d.cyclic == True:
            return self._vector_second_selectivity( d )
        else:
            return self._relative_second_selectivity( d )


    def _relative_second_selectivity( self, d ):
        """
        Return the value of the second maximum as a proportion of the sum_value()
        see _relative_selectivity() for further details
        """
        h   = d._data
        k               = d.max_value_bin()
        v               = h.pop(k)
        m               = max( h.values() )
        h[k]   = v

        proportion      = d._safe_divide( m, sum( h.values() ) )
        offset          = 1.0/len(h)
        scaled          = (proportion-offset)/(1.0-offset)

        return max( scaled, 0.0 )


    def _vector_second_selectivity( self, d ):
        """
        Return the magnitude of the vector_sum() of all bins excluding the
        maximum one, divided by the sum_value().
        see _vector_selectivity() for further details
        """
        h   = d._data
        k               = d.max_value_bin()
        v               = h.pop(k)
        s               = d.vector_sum()[0]
        h[k]   = v

        return self._safe_divide( s, sum( h.values() ) )


    def second_peak_bin( self, d ):
        """
        Return the bin with the second peak in the distribution.
        Unlike second_max_value_bin(), it does not return a bin which is the
        second largest value, if laying on a wing of the first peak, the second
        peak is returned only if the distribution is truly multimodal. If it isn't,
        return the first peak (for compatibility with numpy array type, and
        plotting compatibility), however the correspondong selectivity will be
        forced to 0.0
        """
        h   = d._data
        l   = len( h )
        if l <= 1: 
            return h.keys()[ 0 ]

        ks  = h.keys()
        ks.sort()
        ik0 = ks.index( h.keys()[ argmax( h.values() ) ] )
        k0  = ks[ ik0 ]
        v0  = h[ k0 ]

        v   = v0
        k   = k0
        ik  = ik0
        while h[ k ] <= v:
            ik  += 1
            if ik >= l:
                ik      = 0
            if ik == ik0:
                return k0
            v   = h[ k ]
            k   = ks[ ik ]
        ik1 = ik

        v   = v0
        k   = k0
        ik  = ik0
        while h[ k ] <= v:
            ik  -= 1
            if ik < 0:
                ik  = l - 1
            if ik == ik0:
                return k0
            v   = h[ k ]
            k   = ks[ ik ]
        ik2 = ik

        if ik1 == ik2:
            return ks[ ik1 ]

        ik  = ik1
        m   = 0
        while ik != ik2:
            k   = ks[ ik ]
            if h[ k ] > m:
                m   = h[ k ]
                im  = ik
            ik  += 1
            if ik >= l:
                ik  = 0
        
        return ks[ im ]


    def second_peak_selectivity( self, d ):
        """
        Return the selectivity of the second peak in the distribution.

        If the distribution has only one peak, return 0.0, and this value is
        also usefl to discriminate the validity of second_peak_bin()
        """
        h   = d._data
        if len( h ) <= 1: 
            return 0.0

        p1  = d.max_value_bin()
        p2  = self.second_peak_bin( d )
        if p1 == p2:
            return 0.0

        m           = h[ p2 ]
        proportion  = d._safe_divide( m, sum( h.values() ) )
        offset      = 1.0 / len( h )
        scaled      = (proportion - offset) / (1.0 - offset)

        return max( scaled, 0.0 )


    def second_peak( self, d ):
        """
        Return preference and selectivity of the second peak in the distribution.

        It is just the combination of second_peak_bin() and
        second_peak_selectivity(), with the advantage of avoiding a duplicate
        call of second_peak_bin(), if the user is interested in both preference
        and selectivity, as often is the case.
        """
        h   = d._data
        if len( h ) <= 1: 
            return ( h.keys()[ 0 ], 0.0 )

        p1  = d.max_value_bin()
        p2  = self.second_peak_bin( d )
        if p1 == p2:
            return ( p1, 0.0 )

        m           = h[ p2 ]
        proportion  = d._safe_divide( m, sum( h.values() ) )
        offset      = 1.0 / len( h )
        scaled      = (proportion - offset) / (1.0 - offset)

        return ( p2, max( scaled, 0.0 ) )

    
    __abstract = True



class DSF_MaxValue( DescriptiveStatisticFn ):
    """
    Return the peak value of the given distribution

    """

    def __call__( self, d ):
        p   = self.value_scale[ 1 ] * ( d.max_value_bin() + self.value_scale[ 0 ] )
        s   = self.selectivity_scale[ 1 ] * ( self.selectivity( d ) + self.selectivity_scale[ 0 ] )

        return { "": Pref( preference=p, selectivity=s ) }



class DSF_WeightedAverage( DescriptiveStatisticFn ):
    """
    Return the main mode of the given distribution

    The prefence value ia a continuous, interpolated equivalent of the max_value_bin().
    For a cyclic distribution, this is the direction of the vector
    sum (see vector_sum()).
    For a non-cyclic distribution, this is the arithmetic average
    of the data on the bin_axis, where each bin is weighted by its
    value.
    Such a computation will generally produce much more precise maps using
    fewer test stimuli than the discrete method.  However, weighted_average
    methods generally require uniform and full-range sampling, which is not
    always feasible.
    For measurements at evenly-spaced intervals over the full range of
    possible parameter values, weighted_averages are a good measure of the
    underlying continuous-valued parameter preference, assuming that neurons
    are tuned broadly enough (and/or sampled finely enough) that they
    respond to at least two of the tested parameter values.  This method
    will not usually give good results when those criteria are not met, i.e.
    if the sampling is too sparse, not at evenly-spaced intervals, or does
    not cover the full range of possible values.  In such cases
    max_value_bin should be used, and the number of test patterns will
    usually need to be increased instead.

    """

    def __call__( self, d ):
        p   = self.vector_sum( d )[1]  if d.cyclic  else self._weighted_average( d )
        p   = self.value_scale[ 1 ] * ( p + self.value_scale[ 0 ] )
        s   = self.selectivity_scale[ 1 ] * ( self.selectivity( d ) + self.selectivity_scale[ 0 ] )

        return { "": Pref( preference=p, selectivity=s ) }



class DSF_TopTwoValues( DescriptiveBimodalStatisticFn ):
    """
    Return the two max values of distributions in the given matrix

    """

    def __call__( self, d ):
        r               = {}
        p               = self.value_scale[ 1 ] * ( d.max_value_bin() + self.value_scale[ 0 ] )
        s               = self.selectivity_scale[ 1 ] * ( self.selectivity( d ) + self.selectivity_scale[ 0 ] )
        r[ "" ]         = Pref( preference=p, selectivity=s )
        p               = self.second_max_value_bin( d )
        s               = self.second_selectivity( d )
        p               = self.value_scale[ 1 ] * ( p + self.value_scale[ 0 ] )
        s               = self.selectivity_scale[ 1 ] * ( s + self.selectivity_scale[ 0 ] )
        r[ "Mode2" ]    = Pref( preference=p, selectivity=s )

        return r



class DSF_BimodalPeaks( DescriptiveBimodalStatisticFn ):
    """
    Return the two peak values of distributions in the given matrix

    """

    def __call__( self, d ):
        r               = {}
        p               = self.value_scale[ 1 ] * ( d.max_value_bin() + self.value_scale[ 0 ] )
        s               = self.selectivity_scale[ 1 ] * ( self.selectivity( d ) + self.selectivity_scale[ 0 ] )
        r[ "" ]         = Pref( preference=p, selectivity=s )
        p, s            = self.second_peak( d )
        p               = self.value_scale[ 1 ] * ( p + self.value_scale[ 0 ] )
        s               = self.selectivity_scale[ 1 ] * ( s + self.selectivity_scale[ 0 ] )
        r[ "Mode2" ]    = Pref( preference=p, selectivity=s )

        return r



class VonMisesStatisticFn( DistributionStatisticFn ):
    """
    Base class for von Mises statistics

    """

    # values to fit the maximum value of k parameter in von Mises distribution,
    # as a function of the number of bins in the distribution. Useful for
    # keeping selectivity in range 0..1. Values derived offline from distribution
    # with a single active bin, and total bins from 8 to 32
    vm_kappa_fit    = ( 0.206, 0.614 )

    # level of activity in units confoundable with noise. Used in von Mises fit,
    # for two purposes: if the standard deviation of a distribution is below this
    # value, the distribution is assumed to lack any mode; it is the maximum level
    # of random noise added to a distribution before the fit optimization, for
    # stability reasons
    noise_level     = 0.001

    # exit code of the distribution fit function. Codes are function-specific and
    # each fit function, if provide exit codes, should have corresponding string translation
    fit_exit_code   = 0

    user_warned_if_unavailable = False

    __abstract = True

    def _orth( self, t ):
        """
        Return the orthogonal orientation
        """
        if t < 0.5 * pi:
            return t + 0.5 * pi
        return t - 0.5 * pi


    def _in_pi( self, t ):
        """
        Reduce orientation from -pi..2pi to 0..pi
        """
        if t > pi:
            return t - pi
        if t < 0:
            return t + pi
        return t


    def von_mises( self, pars, x ):
        """
        Compute a simplified von Mises function.

        Original formulation in Richard von Mises, "Wahrscheinlichkeitsrechnung
        und ihre Anwendungen in der Statistik und theoretischen Physik", 1931,
        Deuticke, Leipzig; see also Mardia, K.V. and Jupp, P.E., " Directional
        Statistics", 1999, J. Wiley, p.36;
        http://en.wikipedia.org/wiki/Von_Mises_distribution
        The two differences are that this function is a continuous probability
        distribution on a semi-circle, while von Mises is on the full circle,
        and that the normalization factor, which is the inverse of the modified
        Bessel function of first kind and 0 degree in the original, is here a fit parameter.
        """
        a, k, t = pars
        return a * exp( k * ( cos( 2 * ( x - t ) ) - 1 ) )

    def von2_mises( self, pars, x ):
        """
        Compute a simplified bimodal von Mises function

        Two superposed von Mises functions, with different peak and bandwith values
        """
        p1  = pars[ : 3 ]
        p2  = pars[ 3 : ]
        return self.von_mises( p1, x ) + self.von_mises( p2, x )

    def von_mises_res( self, pars, x, y ):
        return y - self.von_mises( pars, x )

    def von2_mises_res( self, pars, x, y ):
        return y - self.von2_mises( pars, x )

    def norm_sel( self, k, n ):
        m   = ( self.vm_kappa_fit[ 0 ] + n * self.vm_kappa_fit[ 1 ] ) ** 2
        return log( 1 + k ) / log( 1 + m )
        
    def fit_vm( self, distribution ):
        """
        computes the best fit of the monovariate von Mises function in the
        semi-circle.
        Return a tuple with the orientation preference, in the same range of
        axis_bounds, the orientation selectivity, and an estimate of the
        goodness-of-fit, as the variance of the predicted orientation
        preference. The selectivity is given by the bandwith parameter of the
        von Mises function, modified for compatibility with other selectivity
        computations in this class. The bandwith parameter is transposed in
        logaritmic scale, and is normalized by the maximum value for the number
        of bins in the distribution, in order to give roughly 1.0 for a
        distribution with one bin at 1.0 an all the other at 0.0, and 0.0 for
        uniform distributions. The normalizing factor of the selectivity is fit
        for the total number of bins, using fit parameters computed offline.
        There are conditions that prevents apriori the possibility to fit the
        distribution:
            * not enough bins, at least 4 are necessary
            * the distribution is too flat, below the noise level
        and conditions of aposteriori failures:
            * "ier" flag returned by leastsq out of ( 1, 2, 3, 4 )
            * no estimated Jacobian around the solution
            * negative bandwith (the peak of the distribution is convex)
        Note that these are the minimal conditions, their fulfillment does not
        warrant unimodality, is up to the user to check the goodness-of-fit value
        for an accurate acceptance of the fit.
        """
        if unavailable_scipy_optimize:
            if not VonMisesStatisticFn.user_warned_if_unavailable:
                param.Parameterized().warning("scipy.optimize not available, dummy von Mises fit")
                VonMisesStatisticFn.user_warned_if_unavailable=True
            self.fit_exit_code  = 3
            return 0, 0, 0

        to_pi   = pi / distribution.axis_range
        x       = to_pi * numpy.array( distribution.bins() )
        n       = len( x )
        if n < 5:
            param.Parameterized().warning( "no von Mises fit possible with less than 4 bins" )
            self.fit_exit_code  = -1
            return 0, 0, 0

        y   = numpy.array( distribution.values() )
        if y.std() < self.noise_level:
            self.fit_exit_code  = 1
            return 0, 0, 0

        rn  = self.noise_level * numpy.random.random_sample( y.shape )
        p0  = ( 1.0, 1.0, distribution.max_value_bin() )
        r   = optimize.leastsq( self.von_mises_res, p0, args = ( x, y + rn ), full_output = True )

        if not r[ -1 ] in ( 1, 2, 3, 4 ):
            self.fit_exit_code  = 100 + r[ -1 ]
            return 0, 0, 0

        residuals   = r[ 2 ][ 'fvec' ]
        jacobian    = r[ 1 ]
        bandwith    = r[ 0 ][ 1 ]
        tuning      = r[ 0 ][ 2 ]

        if bandwith < 0:
            self.fit_exit_code  = 1
            return 0, 0, 0
            
        if jacobian is None:
            self.fit_exit_code  = 2
            return 0, 0, 0

        error       = ( residuals ** 2 ).sum() / ( n - len( p0 ) )
        covariance  = jacobian * error
        g   = covariance[ 2, 2 ]
        p   = self._in_pi( tuning ) / to_pi
        s   = self.norm_sel( bandwith, n )
        self.fit_exit_code  = 0
        return p, s, g

    def vm_fit_exit_codes ( self ):
        if self.fit_exit_code == 0:
            return "succesfull exit"
        if self.fit_exit_code == -1:
            return "not enough bins for this fit"
        if self.fit_exit_code == 1:
            return "flat distribution"
        if self.fit_exit_code == 2:
            return "flat distribution"
        if self.fit_exit_code == 3:
            return "missing scipy.optimize import"
        if self.fit_exit_code > 110:
            return "unknown exit code"
        if self.fit_exit_code > 100:
            return "error " + str( self.fit_exit_code - 100 ) + " in scipy.optimize.leastsq"

        return "unknown exit code"


    def fit_v2m( self, distribution ):
        """
        computes the best fit of the bivariate von Mises function in the
        semi-circle.
        Return the tuple:
        (
            orientation1_preference, orientation1_selectivity, goodness_of_fit1,
            orientation2_preference, orientation2_selectivity, goodness_of_fit2
        )
        See fit_vm() for considerations about selectivity and goodness_of_fit
        """
        null    = 0, 0, 0, 0, 0, 0
        if unavailable_scipy_optimize:
            if not VonMisesStatisticFn.user_warned_if_unavailable:
                param.Parameterized().warning("scipy.optimize not available, dummy von Mises fit")
                VonMisesStatisticFn.user_warned_if_unavailable=True
            self.fit_exit_code  = 3
            return null

        to_pi   = pi / distribution.axis_range
        x       = to_pi * numpy.array( distribution.bins() )
        n       = len( x )
        if n < 9:
            param.Parameterized().warning( "no bimodal von Mises fit possible with less than 8 bins" )
            self.fit_exit_code  = -1
            return null
        y       = numpy.array( distribution.values() )
        if y.std() < self.noise_level:
            self.fit_exit_code  = 1
            return null

        rn  = self.noise_level * numpy.random.random_sample( y.shape )
        t0  = distribution.max_value_bin()
        p0  = ( 1.0, 1.0, t0, 1.0, 1.0, self._orth( t0 ) )
        r   = optimize.leastsq( self.von2_mises_res, p0, args = ( x, y + rn ), full_output = True )
        if not r[ -1 ] in ( 1, 2, 3, 4 ):
            self.fit_exit_code  = 100 + r[ -1 ]
            return null
        residuals   = r[ 2 ][ 'fvec' ]
        jacobian    = r[ 1 ]
        bandwith_1  = r[ 0 ][ 1 ]
        tuning_1    = r[ 0 ][ 2 ]
        bandwith_2  = r[ 0 ][ 4 ]
        tuning_2    = r[ 0 ][ 5 ]
        if jacobian is None:
            self.fit_exit_code  = 2
            return null
        if bandwith_1 < 0:
            self.fit_exit_code  = 1
            return null
        if bandwith_2 < 0:
            self.fit_exit_code  = 1
            return null

        error       = ( residuals ** 2 ).sum() / ( n - len( p0 ) )
        covariance  = jacobian * error
        g1  = covariance[ 2, 2 ]
        g2  = covariance[ 5, 5 ]
        p1  = self._in_pi( tuning_1 ) / to_pi
        p2  = self._in_pi( tuning_2 ) / to_pi
        s1  = self.norm_sel( bandwith_1, n )
        s2  = self.norm_sel( bandwith_2, n )
        self.fit_exit_code  = 0
        return p1, s1, g1, p2, s2, g2

    def __call__(self,  distribution):
        """
        Apply the distribution statistic function; must be implemented by subclasses.

        """
        raise NotImplementedError



class DSF_VonMisesFit( VonMisesStatisticFn ):
    """
    Return the main mode of distribution in the given matrix, by fit with von Mises function

    """

    worst_fit   = param.Number( default=0.5, bounds=(0.0,None), softbounds=(0.0,1.0), doc="""
            worst good-of-fitness value for accepting the distribution as monomodal""" )

    # default result in case of failure of the fit
    null_result = { "": Pref( preference=0, selectivity=0, goodness_of_fit=0 ), "Modes": Pref( number=0 ) }

    def __call__( self, distribution ):
        f   = self.fit_vm( distribution )
        if self.fit_exit_code != 0 or f[ -1 ] > self.worst_fit:
            return self.null_result
        results             = {}
        p, s, g             = f
        p                   = self.value_scale[ 1 ] * ( p + self.value_scale[ 0 ] )
        s                   = self.selectivity_scale[ 1 ] * ( s + self.selectivity_scale[ 0 ] )
        results[ "" ]       = Pref( preference=p, selectivity=s, goodness_of_fit=g )
        results[ "Modes" ]  = Pref( number=1 )

        return results


class DSF_BimodalVonMisesFit( VonMisesStatisticFn ):
    """
    Return the two modes of distributions in the given matrix, by fit with von Mises function

    The results of the main mode are available in
    self.{preference,selectivity,good_of_fit}, while the second mode results are
    in the first element of the self.more_modes list, as a dictionary with keys
    preference,selectivity,good_of_fit.
    """

    worst_fit   = param.Number( default=0.5, bounds=(0.0,None), softbounds=(0.0,1.0), doc="""
            worst good-of-fitness value for accepting the distribution as mono- or bi-modal""" )

    # default result in case of failure of the fit
    null_result = {
        "":         Pref( preference=0, selectivity=0, goodness_of_fit=0 ),
        "Mode2":    Pref( preference=0, selectivity=0, goodness_of_fit=0 ),
        "Modes":    Pref( number=0 )
    }

    def _analyze_distr( self, d ):
        """
        Analyze the given distribution with von Mises bimodal fit.

        The distribution is analyzed with both unimodal and bimodal fits, and a
        decision about the number of modes is made by comparing the goodness of
        fit. It is a quick but inaccurate way of estimating the number of modes.
        Return preference, selectivity, goodness of fit for both modes, and the
        estimated numer of modes, None if even the unimodal fit failed. If the
        distribution is unimodal, values of the second mode are set to 0. The main
        mode is always the one with the largest selectivity (von Mises bandwith).
        """
        no1 = False
        f   = self.fit_vm( d )
        if self.fit_exit_code != 0:
            no1 = True
        p, s, g = f
        f2  = self.fit_v2m( d )
        if self.fit_exit_code != 0 or f2[ 2 ] > self.worst_fit:
            if no1 or f[ -1 ] > self.worst_fit:
                return None
            return p, s, g, 0, 0, 0, 1
        p1, s1, g1, p2, s2, g2  = f2
        if g1 > g:
            return p, s, g, 0, 0, 0, 1

        if s2 > s1:
            return p2, s2, g2, p1, s1, g1, 2

        return p1, s1, g1, p2, s2, g2, 2


    def __call__( self, distribution ):
        f   = self._analyze_distr( distribution )
        if f is None:
            return self.null_result

        results             = {}
        p, s, g             = f[ : 3 ]
        p                   = self.value_scale[ 1 ] * ( p + self.value_scale[ 0 ] )
        s                   = self.selectivity_scale[ 1 ] * ( s + self.selectivity_scale[ 0 ] )
        results[ "" ]       = Pref( preference=p, selectivity=s, goodness_of_fit=g )
        p, s, g, n          = f[ 3 : ]
        p                   = self.value_scale[ 1 ] * ( p + self.value_scale[ 0 ] )
        s                   = self.selectivity_scale[ 1 ] * ( s + self.selectivity_scale[ 0 ] )
        results[ "Mode2" ]  = Pref( preference=p, selectivity=s, goodness_of_fit=g )
        results[ "Modes" ]  = Pref( number=n )

        return results






"""
Test cases for the boundingregion module.

$Id$
"""

import unittest
import topo
from topo.base.boundingregion import *


class TestAARectangle(unittest.TestCase):
    def setUp(self):
        self.left    = -0.1
        self.bottom  = -0.2
        self.right   =  0.3
        self.top     =  0.4
        self.lbrt = (self.left,self.bottom,self.right,self.top)
        self.aar1 = AARectangle((self.left,self.bottom),(self.right,self.top))
        self.aar2 = AARectangle((self.right,self.bottom),(self.left,self.top))

    def test_left(self):
        self.assertEqual(self.left, self.aar1.left())
    def test_right(self):
        self.assertEqual(self.right, self.aar1.right())
    def test_bottom(self):
        self.assertEqual(self.bottom, self.aar1.bottom())
    def test_top(self):
        self.assertEqual(self.top , self.aar1.top())
    def test_lbrt(self):
        self.assertEqual( self.lbrt, self.aar1.lbrt() )
    def test_point_order(self):
        self.assertEqual( self.aar1.lbrt(), self.aar2.lbrt() )

class TestBoundingBox(unittest.TestCase):
    def setUp(self):
        self.left    = -0.1
        self.bottom  = -0.2
        self.right   =  0.3
        self.top     =  0.4
        self.lbrt = (self.left,self.bottom,self.right,self.top)

        self.region = BoundingBox(points = ((self.left,self.bottom),(self.right,self.top)))
        self.xc,self.yc = self.region.aarect().centroid()

    def test_way_inside(self):
        self.assert_(self.region.contains(0,0))
    def test_above(self):
        self.failIf(self.region.contains(0,1))
    def test_below(self):
        self.failIf(self.region.contains(0,-1))
    def test_left_of(self):
        self.failIf(self.region.contains(-1,0))
    def test_right_of(self):
        self.failIf(self.region.contains(1,0))

    def test_centroid_x(self):
        self.assertEqual(self.xc, (self.left+self.right)/2.0)
    def test_centroid_y(self):
        self.assertEqual(self.yc, (self.bottom+self.top)/2.0)

    def test_left_boundary(self):
        self.assert_(self.region.contains(self.left,self.yc))
    def test_right_boundary(self):
        self.assert_(self.region.contains(self.right,self.yc))
    def test_bottom_boundary(self):
        self.assert_(self.region.contains(self.xc, self.bottom))
    def test_top_boundary(self):
        self.assert_(self.region.contains(self.xc, self.top))
        
                    
class TestBoundingEllipse(TestBoundingBox):
    def setUp(self):
        TestBoundingBox.setUp(self)
        self.region = BoundingEllipse(points = ((self.left,self.bottom),(self.right,self.top)))
    def test_left_top(self):
        self.failIf( self.region.contains(self.left,self.top) )
    def test_right_top(self):
        self.failIf( self.region.contains(self.right,self.top) )
    def test_left_bottom(self):
        self.failIf( self.region.contains(self.left,self.bottom) )
    def test_right_bottom(self):
        self.failIf( self.region.contains(self.right,self.bottom) )

class TestBoundingCircle(TestBoundingEllipse):
    def setUp(self):
        self.xcenter,self.ycenter = (0.2,-0.2)
        self.radius = 0.3
        self.left   =  self.xcenter - self.radius
        self.right  =  self.xcenter + self.radius
        self.bottom =  self.ycenter - self.radius
        self.top    =  self.ycenter + self.radius

        self.region = BoundingCircle(radius = self.radius, center = (self.xcenter,self.ycenter))
        self.xc, self.yc = self.region.aarect().centroid()
        
cases = [TestAARectangle,
         TestBoundingBox,
         TestBoundingEllipse,
         TestBoundingCircle]

suite = unittest.TestSuite()

suite.addTests([unittest.makeSuite(case) for case in cases])
              
if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=1).run(suite)



"""
Inline-optimized Sheet classes

$Id$
"""

import param

from topo.base.cf import MaskedCFIter
from topo.base.projection import NeighborhoodMask
from topo.misc.inlinec import inline,provide_unoptimized_equivalent,c_header
from topo.sheet.lissom import LISSOM
from topo.sheet import compute_joint_norm_totals  # pyflakes:ignore (replaced by optimized version)

def compute_joint_norm_totals_opt(projlist,active_units_mask):
    """
    Compute norm_total for each CF in each projections from a
    group to be normalized jointly.  The same assumptions are
    made as in the original function.
    """
    # Assumes that all Projections in the list have the same r,c size
    length = len(projlist)
    assert length>=1

    proj = projlist[0]
    iterator = MaskedCFIter(proj,active_units_mask=active_units_mask)
    num_cfs = len(proj.flatcfs)  # pyflakes:ignore (passed to weave C code)
    active_units_mask = iterator.get_active_units_mask()
    sheet_mask = iterator.get_sheet_mask()  # pyflakes:ignore (passed to weave C code)
    cf_type = iterator.cf_type  # pyflakes:ignore (passed to weave C code)
    
    # CEBALERT: Not consistent with other C code. E.g. could be
    # simplified to use active_units_mask[] and sheet_mask[]? 

    code = c_header + """
        DECLARE_SLOT_OFFSET(_norm_total,cf_type);
        DECLARE_SLOT_OFFSET(_has_norm_total,cf_type);
        
        npfloat *x = active_units_mask;
        npfloat *m = sheet_mask;
        for (int r=0; r<num_cfs; ++r) {
            double load = *x++;
            double msk = *m++;
            if (msk!=0 && load != 0) {
                double nt = 0;

                for(int p=0; p<length; p++) {
                    PyObject *proj = PyList_GetItem(projlist,p);
                    PyObject *cfs = PyObject_GetAttrString(proj,"flatcfs");
                    PyObject *cf = PyList_GetItem(cfs,r);
                    PyObject *o = PyObject_GetAttrString(cf,"norm_total");
                    nt += PyFloat_AsDouble(o);
                    Py_DECREF(cfs);
                    Py_DECREF(o);
                }

                for(int p=0; p<length; p++) {
                    PyObject *proj = PyList_GetItem(projlist,p);
                    PyObject *cfs = PyObject_GetAttrString(proj,"flatcfs");
                    PyObject *cf = PyList_GetItem(cfs,r);

                    LOOKUP_FROM_SLOT_OFFSET(double,_norm_total,cf);
                    _norm_total[0] = nt;
                    LOOKUP_FROM_SLOT_OFFSET(int,_has_norm_total,cf);
                    _has_norm_total[0] = 1;
                    
                    Py_DECREF(cfs);
                }
            }

        }
    """    
    inline(code, ['projlist','active_units_mask','sheet_mask','num_cfs','length','cf_type'], 
           local_dict=locals(),
           headers=['<structmember.h>'])

provide_unoptimized_equivalent("compute_joint_norm_totals_opt",
                               "compute_joint_norm_totals",locals())

# CEBALERT: not tested
class LISSOM_Opt(LISSOM):
    """
    Faster but potentially unsafe optimized version of LISSOM.

    Adds a NeighborhoodMask that skips computation for neurons
    sufficiently distant from all those activated in the first few
    steps of settling.  This is safe only if activity bubbles reliably
    shrink after the first few steps; otherwise the results will
    differ from LISSOM.

    Typically useful only for standard LISSOM simulations with
    localized (e.g. Gaussian) inputs and that shrink the lateral
    excitatory radius, which results in small patches of activity in
    an otherwise inactive sheet.

    Also overrides the function
    JointNormalizingCFSheet.__compute_joint_norm_totals with
    C-optimized code for LISSOM sheets.
    """
    
    joint_norm_fn = param.Callable(default=compute_joint_norm_totals_opt)

    def __init__(self,**params):
        super(LISSOM_Opt,self).__init__(**params)
        # CEBALERT: this wipes out any user-specified sheet mask.
        self.mask = NeighborhoodMask_Opt(threshold = 0.00001,radius = 0.05,sheet = self)

provide_unoptimized_equivalent("LISSOM_Opt","LISSOM",locals())



class NeighborhoodMask_Opt(NeighborhoodMask):
    
    def calculate(self):
        rows,cols = self.data.shape
        ignore1,matradius = self.sheet.sheet2matrixidx(self.radius,0)
        ignore2,x = self.sheet.sheet2matrixidx(0,0)
        matradius = int(abs(matradius -x))
        thr = self.threshold  # pyflakes:ignore (passed to weave C code)
        activity = self.sheet.activity  # pyflakes:ignore (passed to weave C code)
        mask = self.data  # pyflakes:ignore (passed to weave C code)
        
        code = c_header + """
            #define min(x,y) (x<y?x:y)
            #define max(x,y) (x>y?x:y)
            
            npfloat *X = mask;
            npfloat *A = activity;
            for (int r=0; r<rows; ++r) {
                for (int l=0; l<cols; ++l) {
                    int lbx = max(0,r-matradius);
                    int lby = max(0,l-matradius);
                    int hbx = min(r+matradius+1,rows);
                    int hby = min(l+matradius+1,cols);
                    
                    *X = 0.0;
                    int breakFlag = 0;
                    for(int k=lbx;k<hbx;k++)
                    {
                        for(int l=lby;l<hby;l++)
                        {
                            npfloat *a = A+k*rows + l;
                            if(*a > thr)
                            {
                                *X = 1.0;
                                //JAALERT HACK. Want to jump out both nested loops!!!
                                breakFlag = 1;
                                break;
                            }
                        }
                        if(breakFlag)break;
                    }
                    
                    X++;
                }
            }
        """    
        inline(code, ['thr','activity','matradius','mask','rows','cols'], local_dict=locals())

provide_unoptimized_equivalent("NeighborhoodMask_Opt","NeighborhoodMask",locals())


__all__ = [
    "compute_joint_norm_totals",
    "LISSOM",
    "NeighborhoodMask",
]



"""
Tests for the tkParameterized classes.

$Id$
"""


## CB: add test for change of po


import __main__
import unittest
from Tkinter import Frame,Toplevel

import param
import paramtk as tk

from topo.base.simulation import Simulation
from topo.base.patterngenerator import PatternGenerator
from topo.base.sheet import Sheet

from topo.pattern import Gaussian, Ring, Rectangle
from topo.transferfn import PiecewiseLinear


# CEBALERT: will be replaced with call to param.tk.initialize() when
# param.tk doesn't depend on anything from tkgui.__init__ (plus this
# test will move to the param package...)
import os
if os.getenv("DISPLAY"):
    import topo.tkgui
    topo.tkgui.start()


class SomeFrame(tk.TkParameterized,Frame):
    boo = param.Boolean(default=True)
    osp = param.ObjectSelector()
    csp = param.ClassSelector(class_=PatternGenerator)
    const = param.Parameter(1.0,constant=True)
    pa = param.Parameter(default="test")
    nu = param.Number(default=1.0,bounds=(-1,1))
    st = param.String("string1")

    def __init__(self,master,extraPO=None,**params):
        tk.TkParameterized.__init__(self,master,extraPO=extraPO,**params)
        Frame.__init__(self,master)
        self.boocount=0

    def upboocount(self):
        self.boocount+=1

class TestPO(param.Parameterized):
    bool_param = param.Boolean(default=False)



class TestTkParameterized(unittest.TestCase):


    def setUp(self):
        pass

    def test_basic(self):
        """
        Check basic setting and getting of attributes (via
        object.attribute and GUI set()/get()).
        """
        f = SomeFrame(Toplevel())
        f.pack()

        f.pack_param('const')
        
        f.pack_param('boo',on_set=f.upboocount)

        self.assertEqual(f.boo,True); self.assertEqual(f.boocount,0)

        f.boo = False
        self.assertEqual(f.boocount,1) # check that on_set was called
        self.assertEqual(f.boo,False)  # check that f.boo was set
        self.assertEqual(f._tkvars['boo'].get(),False) # simulate GUI get & check result


        f._tkvars['boo'].set(True) # simulate GUI set
        self.assertEqual(f.boocount,2) # check that on_set was called
        self.assertEqual(f.boo,True) # check that f.boo was actually set
        self.assertEqual(f._tkvars['boo'].get(),True) # simulate GUI get

          
        
    def test_basic_shadow(self):
        """
        Check shadowing of a PO's parameters.
        """
        t = TestPO()
        f = SomeFrame(Toplevel(),extraPO=t)
        f.pack()

        f.pack_param('bool_param') # (from the TestPO)
        
        # test that setting f.bool_param (where bool_param is
        # shadow of t.bool_param) actually sets t.bool_param
        self.assertEqual(t.bool_param,False) # so we know start state
        f.bool_param=True
        self.assertEqual(t.bool_param,True)

        # simulate a GUI set (typing then Return) & then and check
        # g.bool_param is set
        f._tkvars['bool_param'].set(False)
        self.assertEqual(t.bool_param,False) 

        # check that up-to-date value is returned when g got changed elsewhere
        t.bool_param=True
        self.assertEqual(f.bool_param,True) 

        # simulate a GUI get & check up-to-date value returned
        t.bool_param=False
        self.assertEqual(f._tkvars['bool_param'].get(),False) 


        # CEB: the following doesn't work:
        #  self.assertRaises(AttributeError,f.does_not_exist)
        # An AttributeError *is* raised, but it doesn't seem to be
        # caught by testing mechanism. Below is equivalent test...
        try:
            f.does_not_exist
        except AttributeError:
            pass
        else:
            raise("Failed to raise AttributeError on looking up non-existent attribute 'does_not_exist'")
            
        f.did_not_exist = 9
        assert 'did_not_exist' in f.__dict__ # check that new attribute added to f's dict...
        self.assertEqual(f.did_not_exist,9)  # ...and that it was set





    def test_on_set(self):

        f = SomeFrame(Toplevel())

        self.z = 0
        f.pack_param('boo',on_set=self.upzcount)
        f.boo=True
        self.assertEqual(self.z,1)
        
        f._tkvars['boo'].set(True)
        self.assertEqual(self.z,2)


    def test_on_modify(self):

        f = SomeFrame(Toplevel(),boo=True)

        self.z = 0
        f.pack_param('boo',on_modify=self.upzcount)

        f._tkvars['boo'].set(True)
        self.assertEqual(self.z,0)
        
        f._tkvars['boo'].set(False)
        self.assertEqual(self.z,1)
        

    def upzcount(self): self.z+=1  # minor helper method
        


    def test_direct_getting_and_setting(self):
        """
        Test the methods used to set and get Parameter values.
        """
        
        g = Gaussian()
        f = SomeFrame(Toplevel(),extraPO=g) 

        g.size=0.95
        self.assertNotEqual(f.size,g.size)
        self.assertEqual(f.get_parameter_value('size'),g.size)

        f.set_parameter_value('size',0.23)
        self.assertEqual(g.size,0.23)

        try:
            f.set_parameter_value('does_not_exist',100)
        except AttributeError:
            pass
        else:
            raise("Failed to raise AttributeError on setting non-existent *Parameter* 'does_not_exist'")

        try:
            f.get_parameter_value('does_not_exist')
        except AttributeError:
            pass
        else:
            raise("Failed to raise AttributeError on getting non-existent *Parameter* 'does_not_exist'")



class TestParameterTypeRepresentations(unittest.TestCase):


##         In the GUI, objects must sometimes be represented by strings (e.g. for an OptionMenu):
##         test that such conversion is handled correctly


    def setUp(self):
        self.f = SomeFrame(Toplevel())
        import __main__
        __main__.__dict__['f']=self.f


    def test_number_parameter(self):

        self.f.pack_param('nu')
        
        nu_param = self.f.get_parameter_object('nu')
        nu_tkvar = self.f._tkvars['nu']
        nu_widget = self.f.representations['nu']['widget']
        
        # check standard gui setting (includes resolution test)
        self.f.nu = 0.1
        nu_tkvar.set(0.9999999)
        self.f.update_idletasks()
        self.f.update() # seems to be required on OS X
        nu_widget._tag_press_return()
        self.assertEqual(self.f.nu,0.9999999)

        # ...and check the slider moved
        self.assertEqual(nu_widget.slider.get(),0.9999999)

        # check setting the slider
        # CB: note that we can't test actually moving the
        # slider with the mouse...
        nu_widget.slider.set(0.8)
        nu_widget._slider_used()
        self.assertEqual(self.f.nu,0.8)        
        self.assertEqual(nu_widget.slider.get(),0.8)
        self.assertEqual(eval(nu_widget.tag.get()),eval('0.8'))
        self.assertEqual(eval(nu_tkvar.get()),eval('0.8'))

        # now make immediate and test that
        self.f.param_immediately_apply_change[param.Number]=True        
        self.f.nu = 0.1
        nu_tkvar.set(0.4)
        self.assertEqual(self.f.nu,0.4)

        self.f.param_immediately_apply_change[param.Number]=False

        # check gui getting
        self.f.nu = 0.2
        self.assertEqual(eval(nu_tkvar.get()),eval('0.2'))

        # try to set tag beyond the 1 upper bound
        nu_tkvar.set(40.0)
        nu_widget._tag_press_return()
        self.assertEqual(self.f.nu,1) # stays at old value

        # check we can retrieve a value from main
        __main__.__dict__['testA'] = 0.075
        nu_widget.tag.delete(0,"end")
        nu_widget.tag.insert(0,"testA")
        nu_widget._tag_press_return()
        self.assertEqual(self.f.nu,0.075)
        self.assertEqual(nu_widget.tag.get(),'testA') # variable name not overwritten

        # check we can do some basic maths
        exec "from math import sin,pi" in __main__.__dict__
        nu_widget.tag.delete(0,"end")
        nu_widget.tag.insert(0,"sin(pi)")
        nu_widget._tag_press_return()
        from math import sin,pi
        self.assertAlmostEqual(self.f.nu,sin(pi))
        
        

    def test_object_selector_parameter(self):
        """
        Test that ObjectSelectorParameter representation works.        
        """
        some_pos = [param.Parameterized(name='cat'),
                    param.Parameterized(name='rat'),
                    param.Parameterized(name='bat')]
        osp_param = self.f.get_parameter_object('osp')

        osp_param.objects = some_pos
        #self.f.r.default = some_pos[0]

        self.f.pack_param('osp')  # have to pack AFTER populating range for OptionMenu widget to work (see ALERT in tkparameterizedobject.py)

        
        # (otherwise, could do the following:
##         f = SomeFrame(Toplevel())
##         f.pack_param('r')
##         f.initialize_ranged_parameter('r',
##                                       [Parameterized(name='cat'),Parameterized(name='rat'),Parameterized(name='bat')])

        self.assertEqual(self.f.translators['osp'].string2object('cat'),some_pos[0])
        self.assertEqual(self.f.translators['osp'].string2object('rat'),some_pos[1])
        self.assertEqual(self.f.translators['osp'].string2object('bat'),some_pos[2])

        gnat = param.Parameterized(name='gnat')
        osp_param.objects.append(gnat)

        self.f.unpack_param('osp')
        self.f.pack_param('osp') # again, note the need to pack after updating range.
##         self.f.initialize_ranged_parameter('r',Parameterized)
        self.assertEqual(self.f.translators['osp'].string2object('gnat'),gnat)

        self.assertEqual(self.f._object2string('osp',some_pos[0]),'cat')
        self.assertEqual(self.f._object2string('osp',some_pos[1]),'rat')
        self.assertEqual(self.f._object2string('osp',some_pos[2]),'bat')


        # Simulate a GUI set
        self.f._tkvars['osp'].set('bat')
        self.assertEqual(self.f.osp,some_pos[2])

        # Test sorting
        # CB: add


    def test_class_selector_parameter(self):
        self.f.pack_param('csp')

        csp_param = self.f.get_parameter_object('csp')
        csp_tkvar = self.f._tkvars['csp']

        # should maintain instances once created (allows persistence
        # of edits to properties of the classes during the lifetime of
        # a window).

        csp_tkvar.set('Ring')
        self.assertEqual(type(self.f.csp),Ring)
        ring_id = id(self.f.csp)
        
        csp_tkvar.set('Rectangle')
        self.assertEqual(type(self.f.csp),Rectangle)
        rectangle_id = id(self.f.csp)
        
        csp_tkvar.set('Ring')
        self.assertEqual(id(self.f.csp),ring_id)
        csp_tkvar.set('Rectangle')
        self.assertEqual(id(self.f.csp),rectangle_id)
        
        # test sorting
        # CB: add instantiation test, switching test, etc



    def test_string_parameter(self):

        def test_fn(param_name):
            self.f.pack_param(param_name)
            w = self.f.representations[param_name]['widget']

            test_string = "new test string"
            w.delete(0,"end")
            w.insert(0,test_string)
            self.f._update_param_from_tkvar(param_name,force=True)
            
            self.assertEqual(getattr(self.f,param_name),test_string)
            self.assertEqual(w.get(),test_string)

        # unlike when typed by a user, everything's immediate when simulating set()
        self.f.param_immediately_apply_change[param.String] = True
        test_fn('st')

        # and now check the test actually works by making sure
        # it doesn't work for a Parameter (there'll be an extra
        # pair of quotes, stopping a match)
        try:
            test_fn('pa')
        except AssertionError:
            pass
        else:
            raise("Test should fail for Parameter")


# CB: need to add test
##     def test_boolean_parameter(self):

##         self.f.pack_param('boo')
        
        


    def test_parameter(self):
        self.f.pack_param('pa')
        w = self.f.representations['pa']['widget']
        w.delete(0,"end")
        w.insert(0,"AnObjectNamedThisWillNotExist")
        self.f._update_param_from_tkvar('pa',True)
        self.assertEqual(self.f.pa,'test') # didn't get set to wrong value

        # Check that we can create an object from a class in __main__
        exec "from topo.transferfn import IdentityTF" in __main__.__dict__
        w.delete(0,"end")
        w.insert(0,"IdentityTF()")
        self.f._update_param_from_tkvar('pa',force=True)
        import topo.transferfn
        self.assertEqual(type(self.f.pa),topo.transferfn.IdentityTF)




###########################################################

cases = [TestTkParameterized,TestParameterTypeRepresentations]

suite = unittest.TestSuite()
suite.addTests([unittest.makeSuite(case) for case in cases])
suite.requires_display = True

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)




"""
A class for controlling pan/tilt and pan/tilt reset of a camera.
In order to move the camera, you need to install UVCDYNCTRL,
it is a Logitech software for webcam.

This class was tested with uvcdynctrl and Ubuntu 10.04 LTS.


$Id$
"""



from param import Parameterized
import subprocess

UVCDYNCTRLEXEC='/usr/bin/uvcdynctrl'

# Max Ranges (determined with uvcdynctrl -v -c):
# Tilt = -1920 to 1920
# Pan  = -4480 to 4480

def disp(args):
    print " ".join(args)


class PTZ(Parameterized):

    ## Define the init script to initialize the application
    def __init__(self,**params):
        super(PTZ,self).__init__(**params)
	self.reset()

    #Functions for control Pan/tilt
    def tilt(self,value):
        control = "Tilt (relative)"
        disp([UVCDYNCTRLEXEC, "-s", control, "--", str(value)])
        subprocess.call([UVCDYNCTRLEXEC, "-s", control, "--", str(-value)])

    def pan(self, value):
        control = "Pan (relative)"
        disp([UVCDYNCTRLEXEC, "-s", control, "--", str(value)])
        subprocess.call([UVCDYNCTRLEXEC, "-s", control, "--", str(-value)])

    def reset(self):
        value = "3"
        control = "Pan Reset"
        disp([UVCDYNCTRLEXEC, "-s", control, value])
        subprocess.call([UVCDYNCTRLEXEC, "-s", control, value])
        control = "Tilt Reset"
        disp([UVCDYNCTRLEXEC, "-s", control, value])
        subprocess.call([UVCDYNCTRLEXEC, "-s", control, value])
        control = "Pan/tilt Reset"
        disp([UVCDYNCTRLEXEC, "-s", control, value])
        subprocess.call([UVCDYNCTRLEXEC, "-s", control, value])



from numpy import zeros

from topo.base.projection import SheetMask,ProjectionSheet

class LeftColMask(SheetMask):

    def reset(self):
        self._data = zeros(self.sheet.shape)
        self._data[:,0] += 1

class RightColMask(SheetMask):

    def reset(self):
        self._data = zeros(self.sheet.shape)
        self._data[:,-1] += 1

class TopRowMask(SheetMask):

    def reset(self):
        self._data = zeros(self.sheet.shape)
        self._data[0,:] += 1

class BottomRowMask(SheetMask):

    def reset(self):
        self._data = zeros(self.sheet.shape)
        self._data[-1,:] += 1
        



"""
Unit tests for sheet and sheetcoords.

$Id$
"""

import unittest
from numpy import array

from topo.base.sheetcoords import SheetCoordinateSystem,Slice
from topo.base.sheet import *
from topo.base.boundingregion import BoundingBox
from topo.base.sheetview import SheetView


# CEBALERT: 
# Changes that need to be made to this file:
# - don't create a new ct every time, just change its density
# - test array versions of coordinate transform functions
# - ensure methods of Slice are tested


class TestCoordinateTransforms(unittest.TestCase):
    """
    Tests for sheet.

    Subclassed for use; the subclasses have a setUp method to create
    the particular coordinates to use each time.
    """    
    def makeBox(self):
        self.box = BoundingBox(points=((self.left,self.bottom),
                                       (self.right,self.top)))

        self.ct = SheetCoordinateSystem(self.box,self.density,self.density)

        # float bounds for matrix coordinates: these
        # values are actually outside the matrix
        self.rbound = self.density*(self.top-self.bottom)
        self.cbound = self.density*(self.right-self.left)

        #self.cbound = int(self.density*(self.right-self.left)) / float((self.right-self.left))
        #self.rbound = int(self.density*(self.top-self.bottom)) / float((self.top-self.bottom))


        # CEBALERT: this is supposed to be a small distance
        D = 0.00001

        # Sheet values around the edge of the BoundingBox
        self.just_in_right_x = self.right - D
        self.just_in_bottom_y = self.bottom + D
        self.just_out_top_y = self.top + D
        self.just_out_left_x = self.left - D

        # Matrix values around the edge of the matrix
        self.just_out_right_idx = self.rbound + D
        self.just_out_bottom_idx = self.cbound + D
        self.just_out_top_idx = 0.0 - D
        self.just_out_left_idx = 0.0 - D


        
    ### sheet2matrix() tests
    #
    def test_sheet2matrix_center(self):
        """
        Check that the center of the Sheet corresponds to the center
        of the matrix.
        """
        x_center = self.left+(self.right-self.left)/2.0
        y_center = self.bottom+(self.top-self.bottom)/2.0
        row, col = self.ct.sheet2matrix(x_center,y_center)
        self.assertEqual((row,col),(self.rbound/2.0,self.cbound/2.0))


    def test_sheet2matrix_left_top(self):
        """
        Check that the top-left of the Sheet is [0,0] in matrix
        coordinates.
        """
        row, col = self.ct.sheet2matrix(self.left,self.top)
        self.assertEqual((row,col),(0,0))


    def test_sheet2matrix_right_bottom(self):
        """
        Check that the bottom-right of the Sheet is [rbound,cbound] in matrix
        coordinates.
        """
        row, col = self.ct.sheet2matrix(self.right,self.bottom)
        self.assertEqual((row,col),(self.rbound,self.cbound))

        
    def test_sheet2matrix_matrix2sheet(self):
        """
        Check that matrix2sheet() is the inverse of sheet2matrix().
        """
        # top-right corner
        row, col = self.ct.sheet2matrix(self.right,self.top)
        x_right, y_top = self.ct.matrix2sheet(row,col)
        self.assertEqual((x_right,y_top),(self.right,self.top)) 

        # bottom-left corner
        row, col = self.ct.sheet2matrix(self.left,self.bottom)
        x_left, y_bottom = self.ct.matrix2sheet(row,col)
        self.assertEqual((x_left,y_bottom),(self.left,self.bottom)) 


    def test_matrix2sheet_sheet2matrix(self):
        """
        Check that sheet2matrix() is the inverse of matrix2sheet().
        """
        # top-right corner
        x,y = self.ct.matrix2sheet(float(0),float(self.last_col))
        top_row,right_col = self.ct.sheet2matrix(x,y)
        self.assertEqual((top_row,right_col),(float(0),float(self.last_col))) 

        # bottom-left corner
        x,y = self.ct.matrix2sheet(float(self.last_row),float(0))
        bottom_row,left_col = self.ct.sheet2matrix(x,y)
        self.assertEqual((bottom_row,left_col),(float(self.last_row),float(0)))

        
    ### sheet2matrixidx() tests
    #    
    def test_sheet2matrixidx_left_top(self):
        """
        Test a point just inside the top-left corner of the BoundingBox, and
        one just outside.
        """
        # inside
        r,c = 0,0
        x,y = self.left,self.top
        self.assertEqual(self.ct.sheet2matrixidx(x,y), (r,c))

        # outside
        r,c = -1,-1
        x,y = self.just_out_left_x,self.just_out_top_y
        self.assertEqual(self.ct.sheet2matrixidx(x,y), (r,c))


    def test_sheet2matrixidx_left_bottom(self):
        """
        Test a point just inside the left-bottom corner of the BoundingBox, and
        one just outside.
        """
        # inside
        r,c = self.last_row, 0
        x,y = self.left, self.just_in_bottom_y
        self.assertEqual(self.ct.sheet2matrixidx(x,y),(r,c))

        # outside
        r,c = self.last_row+1, -1
        x,y = self.just_out_left_x, self.bottom
        self.assertEqual(self.ct.sheet2matrixidx(x,y),(r,c))


    def test_sheet2matrixidx_right_top(self):
        """
        Test a point just inside the top-right corner of the BoundingBox, and
        one just outside.
        """
        # inside
        r,c = 0,self.last_col
        x,y = self.just_in_right_x,self.top
        self.assertEqual(self.ct.sheet2matrixidx(x,y),(r,c))

        # outside
        r,c = -1,self.last_col+1
        x,y = self.right,self.just_out_top_y
        self.assertEqual(self.ct.sheet2matrixidx(x,y),(r,c))


    def test_sheet2matrixidx_right_bottom(self):
        """
        Test a point just inside the bottom-right corner of the BoundingBox,
        and the corner itself - which should not be inside.
        """
        # inside 
        r,c = self.last_row,self.last_col
        x,y = self.just_in_right_x,self.just_in_bottom_y
        self.assertEqual(self.ct.sheet2matrixidx(x,y),(r,c))

        # not inside
        r,c = self.last_row+1,self.last_col+1
        x,y = self.right,self.bottom
        self.assertEqual(self.ct.sheet2matrixidx(x,y),(r,c))


    ### matrix2sheet() tests
    #
    def test_matrix2sheet_left_top(self):
        """
        Check that Sheet's (0,0) is the top-left of the matrix.

        Check that just outside the top-left in matrix coordinates
        comes back to Sheet coordinates that are outside the
        BoundingBox.
        """
        x,y = self.ct.matrix2sheet(0,0)
        self.assertEqual((x,y), (self.left,self.top))

        x,y = self.ct.matrix2sheet(self.just_out_left_idx,self.just_out_top_idx)
        self.assertFalse(self.box.contains(x,y))
    

    def test_matrix2sheet_right_bottom(self):
        """
        Check that Sheet's (right,bottom) is the bottom-right in
        matrix coordinates i.e. [rbound,cbound]

        Check that just outside the bottom-right in matrix coordinates
        comes back to Sheet coordinates that are outside the
        BoundingBox.
        """
        x,y = self.ct.matrix2sheet(self.rbound,self.cbound)
        self.assertEqual((x,y), (self.right,self.bottom))

        x,y = self.ct.matrix2sheet(self.just_out_right_idx,self.just_out_bottom_idx)
        self.assertFalse(self.box.contains(x,y))


    def test_matrix2sheet_center(self):
        """
        Check that the center in Sheet coordinates corresponds to
        the center in continuous matrix coordinates.
        """
        x_center = self.left+(self.right-self.left)/2.0
        y_center = self.bottom+(self.top-self.bottom)/2.0
        center_float_row = self.rbound/2.0
        center_float_col = self.cbound/2.0
        x,y = self.ct.matrix2sheet(center_float_row,center_float_col)
        self.assertEqual((x,y),(x_center,y_center))



    ### matrixidx2sheet() tests
    #
    def test_matrixidx2sheet_left_top(self):
        """
        The top-left matrix cell [0,0] should be given back in Sheet
        coordinates at the center of that cell.

        The cell [-1,-1] outside this corner should come back out of
        the BoundingBox
        """
        # inside
        r,c = 0,0
        x,y = self.left+self.half_unit,self.top-self.half_unit

        test_x, test_y = self.ct.matrixidx2sheet(r,c)
        self.assertEqual((test_x,test_y), (x,y))
        self.assertTrue(self.box.contains(test_x,test_y))

        # outside
        r,c = -1,-1
        test_x, test_y = self.ct.matrixidx2sheet(r,c)
        self.assertFalse(self.box.contains(test_x,test_y))
        
        
    def test_matrixidx2sheet_left_bottom(self):
        """
        The bottom-left matrix cell [0,rbound] should be given back
        in Sheet coordinates at the center of that cell.

        The cell [last_row+1,-1] outside this corner should come back out of
        the BoundingBox.
        """
        # inside
        r,c = self.last_row,0
        x,y = self.left+self.half_unit,self.bottom+self.half_unit
        self.assertEqual(self.ct.matrixidx2sheet(r,c), (x,y))

        # outside
        r,c = self.last_row+1,-1
        test_x, test_y = self.ct.matrixidx2sheet(r,c)
        self.assertFalse(self.box.contains(test_x,test_y))

        
    def test_matrixidx2sheet_right_top(self):
        """
        The top-right matrix cell [cbound,0] should be given back
        in Sheet coordinates at the center of that cell.

        The cell [-1,last_col+1] outside this corner should come back out of
        the BoundingBox.
        """
        # inside
        r,c = 0,self.last_col
        x,y = self.right-self.half_unit,self.top-self.half_unit
        self.assertEqual(self.ct.matrixidx2sheet(r,c), (x,y))

        # outside
        r,c = -1,self.last_col+1
        test_x, test_y = self.ct.matrixidx2sheet(r,c)
        self.assertFalse(self.box.contains(test_x,test_y))

        
    def test_matrixidx2sheet_right_bottom(self):
        """
        The bottom-right matrix cell [cbound,rbound] should be given back
        in Sheet coordinates at the center of that cell.

        The cell [last_row+1,last_col+1] outside this corner should come back out of
        the BoundingBox.
        """
        r,c = self.last_row,self.last_col
        x,y = self.right-self.half_unit,self.bottom+self.half_unit
        self.assertEqual(self.ct.matrixidx2sheet(r,c), (x,y))

        # outside
        r,c = self.last_row+1,self.last_col+1
        test_x, test_y = self.ct.matrixidx2sheet(r,c)
        self.assertFalse(self.box.contains(test_x,test_y))


    def test_matrixidx2sheet_center(self):
        """
        The row and col *index* of the center unit in the matrix should come
        back as the Sheet coordinates of the center of that center unit.
        """
        r,c = self.center_unit_idx
        x_center = self.left+(self.right-self.left)/2.0
        y_center = self.bottom+(self.top-self.bottom)/2.0
        x,y = x_center+self.half_unit, y_center-self.half_unit
        self.assertEqual(self.ct.matrixidx2sheet(r,c), (x,y))    

    def test_matrixidx2sheet_sheet2matrixidx(self):
        """
        Check that sheet2matrixidx() is the inverse of matrix2sheetidx().
        """
        # top-right corner
        x,y = self.ct.matrixidx2sheet(float(0),float(self.last_col))
        top_row,right_col = self.ct.sheet2matrixidx(x,y)
        self.assertEqual((top_row,right_col),(float(0),float(self.last_col))) 

        # bottom-left corner
        x,y = self.ct.matrixidx2sheet(float(self.last_row),float(0))
        bottom_row,left_col = self.ct.sheet2matrixidx(x,y)
        self.assertEqual((bottom_row,left_col),(float(self.last_row),float(0)))




        



class TestBox1Coordinates(TestCoordinateTransforms):
    """
    Test coordinate transformations using the standard, origin-centered unit box
    with density 10.

    A 10x10 matrix.
    """
    def setUp(self):
        self.left = -0.5
        self.bottom = -0.5
        self.top = 0.5
        self.right = 0.5
        self.density = 10
        self.half_unit = 0.05

        # for the matrix representation - I think having this manual statement is
        # safer than a calculation...
        self.last_row = 9
        self.last_col = 9
        self.center_unit_idx = (5,5)  # by the way sheet2matrixidx is defined

        self.makeBox()


class TestBox2Coordinates(TestCoordinateTransforms):
    """
    Test coordinate transformations on the box defined by (1,1), (3,4),
    with density 8.

    A 24 x 16 matrix.
    """
    def setUp(self):
        self.left = 1
        self.bottom = 1
        self.right = 3
        self.top  = 4
        self.density = 8
        self.half_unit = 0.0625

        # for the matrix representation - I think having this manual statement is
        # safer than a calculation...
        self.last_row = 23
        self.last_col = 15
        self.center_unit_idx = (12,8)  # by the way sheet2matrixidx is defined

        self.makeBox()


# CEB: unfinished and unused - still making tests for TestBox3Coordinates...
class TestBox3Coordinates(TestCoordinateTransforms):

    def setUp(self):        
        self.left = -0.8
        self.bottom = -0.8
        self.top = 0.8
        self.right = 0.8
        self.density = 16
        self.half_unit = 0.03125

        # for the matrix representation - I think having this manual statement is
        # safer than a calculation...
        self.last_row = 24
        self.last_col = 24
        self.center_unit_idx = (12,12)  # by the way sheet2matrixidx is defined

        self.makeBox()



suite = unittest.TestSuite()

cases = [TestBox1Coordinates,TestBox2Coordinates]#,TestBox3Coordinates]
suite.addTests([unittest.makeSuite(case) for case in cases])



class ExtraSheetTests(unittest.TestCase):
    """
    sheet tests that were written independently of the framework above.

    Each of these tests runs once and is independent of the rest of the file.
    """

    def test_slice2bounds_bounds2slice(self):

        bb = BoundingBox(points=((-0.5,-0.5),(0.5,0.5)))
        ct = SheetCoordinateSystem(bb,10)

        slice_ =(0,3,7,8)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        slice_ =(4,7,8,10)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        slice_ =(2,3,4,8)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        slice_ =(0,3,9,10)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        bb = BoundingBox(points=((-0.75,-0.5),(0.75,0.5)))
        ct = SheetCoordinateSystem(bb,20,20)

        slice_ =(9,14,27,29)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        slice_ =(0,6,0,7)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        slice_ =(6,10,11,29)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        bb = BoundingBox(points=((-0.5,-0.5),(0.5,0.5)))
        ct = SheetCoordinateSystem(bb,7)

        slice_ =(4,7,2,3)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)

        slice_ =(0,7,0,7)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        test_slice = Slice(bounds,ct)

        for a,b in zip(slice_,test_slice):
            self.assertEqual(a,b)


    def test_coordinate_position(self):
        """
        these tests duplicate some of the earlier ones,
        except these use a matrix with non-integer
        (right-left) and (top-bottom). This is an important
        test case for the definition of density; without it,
        the tests above could be passed by a variety of
        sheet2matrix, bounds2shape functions, etc.
        
        CEBALERT: transfer the box to TestBox3Coordinates and have
        these tests run in the framework.
        """
        l,b,r,t = (-0.8,-0.8,0.8,0.8)
        # mimics that a sheet recalculates its density)
        density = int(16*(r-l)) / float(r-l)

        bounds = BoundingBox(points=((l,b),(r,t)))

        ct = SheetCoordinateSystem(bounds,density,density)
        
        self.assertEqual(ct.sheet2matrixidx(0.8,0.8),(0,24+1))
        self.assertEqual(ct.sheet2matrixidx(0.0,0.0),(12,12))
        self.assertEqual(ct.sheet2matrixidx(-0.8,-0.8),(24+1,0))
        self.assertEqual(ct.matrixidx2sheet(24,0),
                         (((r-l) / int(density*(r-l)) / 2.0) + l,
                          (t-b) / int(density*(t-b)) / 2.0 + b))
        self.assertEqual(ct.matrixidx2sheet(0,0),
                         (((r-l) / int(density*(r-l)) / 2.0) + l ,
                          (t-b) / int(density*(t-b)) * (int(density*(t-b)) - 0.5) + b))

        x,y = ct.matrixidx2sheet(0,0)
        self.assertTrue(bounds.contains(x,y))
        self.assertEqual((0,0),ct.sheet2matrixidx(x,y))

        x,y = ct.matrixidx2sheet(25,25)
        self.assertFalse(bounds.contains(x,y))
        self.assertNotEqual((24,24),ct.sheet2matrixidx(x,y))

        x,y = ct.matrixidx2sheet(0,24)
        self.assertTrue(bounds.contains(x,y))
        self.assertEqual((0,24),ct.sheet2matrixidx(x,y))

        x,y = ct.matrixidx2sheet(24,0)
        self.assertTrue(bounds.contains(x,y))
        self.assertEqual((24,0),ct.sheet2matrixidx(x,y))


    def test_Sheet_creation(self):

        # Example where the nominal x and y densities would not be equal.
        # density along x =10
        # density along y <10
        # The density along y should become 10, by adjusting the height to be 2.0
        # in this case.
        # The y center of the bounds should remain -0.0025. Hence we should get
        # a bottom bound of -1.0025 and a top one of 0.9975.
        sheet = Sheet(nominal_density=10,
                      nominal_bounds=BoundingBox(points=((-0.5,-1.005),(0.5,1.0))))

        self.assertEqual(sheet.xdensity,10)
        self.assertEqual(sheet.xdensity,sheet.ydensity)

        l,b,r,t = sheet.lbrt
        self.assertEqual(l,-0.5)
        self.assertEqual(r,0.5)                      
        self.assertAlmostEqual(t,0.9975)
        self.assertAlmostEqual(b,-1.0025)



    # CEBALERT: this test should probably be somewhere else and
    # called something different
    def test_connection_field_like(self):
        # test a ConnectionField-like example
        sheet = Sheet(nominal_density=10,nominal_bounds=BoundingBox(radius=0.5))
        cf_bounds = BoundingBox(points=((0.3,0.3),(0.6,0.6)))

        slice_ = Slice(cf_bounds,sheet)
        slice_.crop_to_sheet(sheet)

        # check it's been cropped to fit onto sheet...
        self.assertEqual(slice_.tolist(),[0,2,8,10])

        # now check that it gives the correct bounds...
        cropped_bounds = slice_.compute_bounds(sheet)
        
        true_cropped_bounds = BoundingBox(points=((0.3,0.3),(0.5,0.5)))
        for a,b in zip(cropped_bounds.lbrt(),true_cropped_bounds.lbrt()):
            self.assertAlmostEqual(a,b)

        # and that bounds2shape() gets the correct size
#        rows,cols = bounds2shape(cropped_bounds,sheet.density,sheet.ydensity)
#        self.assertEqual((rows,cols),(2,2))


    def test_bounds2slice(self):
        
        # test that if you ask to slice the matrix with the sheet's BoundingBox, you
        # get back the whole matrix
        sheet_bb = BoundingBox(points=((-0.5,-0.5),(0.5,0.5)))
        ct = SheetCoordinateSystem(sheet_bb,10)
        
        slice_ = Slice(sheet_bb,ct)
        true_slice = (0,10,0,10) # inclusive left boundary, exclusive right boundary
        self.assertEqual(tuple(slice_.tolist()),true_slice) 

        # for the following tests, the values have been all computed by hand and then
        # tested (by JC). The boundingbox and density tested have been chosen randomly,
        # then drawn to get the slice from it.
       
        # Test with 20 density. 
        ct = SheetCoordinateSystem(sheet_bb,20,20)
        bb = BoundingBox(points=((-0.05,-0.20),(0.20,0.05)))
        slice_ = Slice(bb,ct)

        true_slice = (9,14,9,14) 
        self.assertEqual(tuple(slice_.tolist()),true_slice)

        bb = BoundingBox(points=((-0.40,0),(-0.30,0.30)))
        slice_ = Slice(bb,ct)
        true_slice = (4,10,2,4) 
        self.assertEqual(tuple(slice_.tolist()),true_slice)

        bb = BoundingBox(points=((0.15,0.10),(0.30,0.30)))
        slice_ = Slice(bb,ct)
        true_slice = (4,8,13,16) 
        self.assertEqual(tuple(slice_.tolist()),true_slice)

        bb = BoundingBox(points=((-0.05,-0.45),(0.10,-0.25)))
        slice_ = Slice(bb,ct)
        true_slice = (15,19,9,12) 
        self.assertEqual(tuple(slice_.tolist()),true_slice)
        
        # test with 7 density sheet.
        
        bb = BoundingBox(points=((-0.5+2.0/7.0,0.5-2.0/7.0),(-0.5+4.0/7.0,0.5)))
        ct = SheetCoordinateSystem(sheet_bb,7)
        
        slice_ = Slice(bb,ct)
        true_slice = (0,2,2,4) 
        self.assertEqual(tuple(slice_.tolist()),true_slice)

        #(4x4 matrix)
        ct = SheetCoordinateSystem(BoundingBox(radius=0.2),xdensity=10,ydensity=10)
        test_bounds = BoundingBox(radius=0.1)
        slice_=Slice(test_bounds,ct)
        r1,r2,c1,c2 = slice_
        self.assertEqual((r1,r2,c1,c2),(1,3,1,3))

        # Note: this test fails because units that fall on the
        # boundaries should all be included; bounds2slice does not
        # include the ones on the left boundary because of floating point
        # representation.
        #test_bounds.translate(0.05,-0.05)
        #r1,r2,c1,c2 = ct.bounds2slice(test_bounds)
        #self.assertEqual((r1,r2,c1,c2),(1,4,1,4))
        
        
    def test_slice2bounds(self):

        # test that if you ask to slice the matrix with the sheet's BoundingBox, you
        # get back the whole matrix 
        # (I chose to use a 7 density, I don't know why I like 7 so much, it is kind of mystical)
        
        sheet_bb = BoundingBox(points=((-0.5,-0.5),(0.5,0.5)))
        ct = SheetCoordinateSystem(sheet_bb,7)
        slice_ = (0,7,0,7)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        true_bounds_lbrt = (-0.5,-0.5,0.5,0.5)
        for a,b in zip(bounds.lbrt(),true_bounds_lbrt):
            self.assertAlmostEqual(a,b)

        # for the following tests, the values have been all computed
        # by hand and then tested (by JC). The boundingbox and density
        # tested have been chosen randomly, then drawn to get the slice
        # from it.
        
        # Test for 10 density
        ct = SheetCoordinateSystem(sheet_bb,10)
        slice_ = (0,9,1,5)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        true_bounds_lbrt = (-0.4,-0.4,0,0.5)
        for a,b in zip(bounds.lbrt(),true_bounds_lbrt):
            self.assertAlmostEqual(a,b)

        slice_ = (2,3,7,10)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        true_bounds_lbrt = (0.2,0.2,0.5,0.3)
        for a,b in zip(bounds.lbrt(),true_bounds_lbrt):
            self.assertAlmostEqual(a,b)
        
        # Test for 7 density
        ct = SheetCoordinateSystem(sheet_bb,7)
        slice_ = (3,7,2,5)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        true_bounds_lbrt = (-0.5+2.0/7.0,-0.5,-0.5+5.0/7.0,0.5-3.0/7.0)
        for a,b in zip(bounds.lbrt(),true_bounds_lbrt):
            self.assertAlmostEqual(a,b)

        slice_ = (2,6,0,1)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        true_bounds_lbrt = (-0.5,0.5-6.0/7.0,-0.5+1.0/7.0,0.5-2.0/7.0)
        for a,b in zip(bounds.lbrt(),true_bounds_lbrt):
            self.assertAlmostEqual(a,b)

        # Test for 25 density
        ct = SheetCoordinateSystem(sheet_bb,25)
        slice_ = (0,25,4,10)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        true_bounds_lbrt = (-0.5+4.0/25.0,-0.5,-0.5+10.0/25.0,0.5)
        for a,b in zip(bounds.lbrt(),true_bounds_lbrt):
            self.assertAlmostEqual(a,b)
        

        slice_ = (7,18,3,11)
        bounds = BoundingBox(points=Slice._slicespec2boundsspec(slice_,ct))
        true_bounds_lbrt = (-0.5+3.0/25.0,0.5-18.0/25.0,-0.5+11.0/25.0,0.5-7.0/25.0)
        for a,b in zip(bounds.lbrt(),true_bounds_lbrt):
            self.assertAlmostEqual(a,b)
            

##     # bounds2shape() tests
##     #
##     def test_bounds2shape(self):
##         """
##         Check that the shape of the matrix based on the BoundingBox and
##         density is correct.
##         """
##         n_rows,n_cols = bounds2shape(self.box,self.density)
##         self.assertEqual((n_rows,n_cols),(self.last_row+1,self.last_col+1))

        
    def test_sheetview_release(self):
        s = Sheet()
        s.activity = array([[1,2],[3,4]])
        # Call s.sheet_view(..) with a parameter
        sv2 = SheetView((s.activity,s.bounds),
                          src_name=s.name)
        self.assertEqual(len(s.sheet_views.keys()),0)
        s.sheet_views['Activity']=sv2
        self.assertEqual(len(s.sheet_views.keys()),1)
        s.release_sheet_view('Activity')
        self.assertEqual(len(s.sheet_views.keys()),0)




suite.addTest(unittest.makeSuite(ExtraSheetTests))






if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)
        



"""
Analysis tools for Topographica, other than plotting tools.

$Id$
"""


# Automatically discover all .py files in this directory.
import os,fnmatch
__all__ = [f.split('.py')[0] for f in os.listdir(__path__[0]) if fnmatch.fnmatch(f,'[!._]*.py')]
del f,os,fnmatch




"""

$Id$
"""

import unittest, copy, shutil, tempfile
from numpy.testing import assert_array_equal

from param import normalize_path,resolve_path
import topo
import __main__

from topo.base.sheet import Sheet
from topo.sheet import GeneratorSheet
from topo.command import save_snapshot,load_snapshot
from topo.pattern import Gaussian, Line
from topo.base.simulation import Simulation,SomeTimer


SNAPSHOT_NAME = "testsnapshot.typ"
SIM_NAME = "testsnapshots"

class TestSnapshots(unittest.TestCase):

    # CB: all tests that use topo.sim ought to do make a new topo.sim
    def setUp(self):
        """
        Create a new Simulation as topo.sim (so this test isn't affected by changes
        to topo.sim by other tests).
        """
        Simulation(register=True,name=SIM_NAME)
        self.original_output_path = normalize_path.prefix
        normalize_path.prefix = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(normalize_path.prefix)
        normalize_path.prefix=self.original_output_path
        
    def basic_save_load_snapshot(self):
        """
        Very basic test to check the activity matrix of a GeneratorSheet
        comes back ok, and that class attributes are pickled.
        """
        assert topo.sim.name==SIM_NAME
         
        topo.sim['R']=GeneratorSheet(input_generator=Gaussian(),nominal_density=2)

        topo.sim.run(1)

        R_act = copy.deepcopy(topo.sim['R'].activity)
        Line.x = 12.0
        topo.sim.startup_commands.append("z=99")

        save_snapshot(SNAPSHOT_NAME)


        Line.x = 9.0
        exec "z=88" in __main__.__dict__
        
        topo.sim['R'].set_input_generator(Line())
        topo.sim.run(1)

        load_snapshot(resolve_path(SNAPSHOT_NAME,search_paths=[normalize_path.prefix]))

        
        # CEBALERT: should also test that unpickling order is correct
        # (i.e. startup_commands, class attributes, simulation)
        assert_array_equal(R_act,topo.sim['R'].activity)
        self.assertEqual(Line.x,12.0)
        self.assertEqual(__main__.__dict__['z'],99)

        

    def test_basic_save_load_snapshot(self):
        self.basic_save_load_snapshot()




    def test_new_simulation_still_works(self):

        #  Test to make sure the above tests haven't screwed up
        # the ability to construct new simulation objects
        topo.base.simulation.Simulation()


# CB: longer to run test should additionally quit the simulation
# and start again. Should also test scheduled actions.
   
        
suite = unittest.TestSuite()
suite.addTest(unittest.makeSuite(TestSnapshots))



"""
Output functions (see basic.py) that apply to a whole Projection. For
example, for a CFProjection this involves iterating through all the
CFs and applying an output function to each set of weights in turn.

$Id$
"""

from topo.base.cf import CFPOutputFn

# imported here so that all projection-level output functions are in the
# same package
from topo.base.cf import CFPOF_Plugin,CFPOF_Identity  # pyflakes:ignore (API import)



__all__ = list(set([k for k,v in locals().items() if isinstance(v,type) and issubclass(v,CFPOutputFn)]))
__all__.remove("CFPOutputFn")
